// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AISS16AO2/driver/16aiss16ao2.h $
// $Rev: 52935 $
// $Date: 2023-04-20 15:51:33 -0500 (Thu, 20 Apr 2023) $

// 16AISS16AO2: Device Driver: header file

#ifndef	__16AISS16AO2_H__
#define	__16AISS16AO2_H__

#include "gsc_common.h"
#include "gsc_pci9056.h"



// macros *********************************************************************

#define	AISS16AO2_BASE_NAME					"16aiss16ao2"

// IOCTL command codes
#define	AISS16AO2_IOCTL_REG_READ			OS_IOCTL_RW( 0, 12, gsc_reg_t)
#define	AISS16AO2_IOCTL_REG_WRITE			OS_IOCTL_W ( 1, 12, gsc_reg_t)
#define	AISS16AO2_IOCTL_REG_MOD				OS_IOCTL_W ( 2, 12, gsc_reg_t)
#define	AISS16AO2_IOCTL_QUERY				OS_IOCTL_RW( 3,  4, s32)
#define	AISS16AO2_IOCTL_INITIALIZE			OS_IOCTL   ( 4)
#define	AISS16AO2_IOCTL_AUTO_CALIBRATE		OS_IOCTL   ( 5)
#define	AISS16AO2_IOCTL_AUTO_CAL_STS		OS_IOCTL_R ( 6,  4, s32)
#define	AISS16AO2_IOCTL_RX_IO_MODE			OS_IOCTL_RW( 7,  4, s32)
#define	AISS16AO2_IOCTL_RX_IO_OVERFLOW		OS_IOCTL_RW( 8,  4, s32)
#define	AISS16AO2_IOCTL_RX_IO_TIMEOUT		OS_IOCTL_RW( 9,  4, s32)
#define	AISS16AO2_IOCTL_RX_IO_UNDERFLOW		OS_IOCTL_RW(10,  4, s32)
#define	AISS16AO2_IOCTL_TX_IO_MODE			OS_IOCTL_RW(11,  4, s32)
#define	AISS16AO2_IOCTL_TX_IO_OVERFLOW		OS_IOCTL_RW(12,  4, s32)
#define	AISS16AO2_IOCTL_TX_IO_TIMEOUT		OS_IOCTL_RW(13,  4, s32)
#define	AISS16AO2_IOCTL_AI_BUF_CLEAR		OS_IOCTL   (14)
#define	AISS16AO2_IOCTL_AI_BUF_LEVEL		OS_IOCTL_R (15,  4, s32)
#define	AISS16AO2_IOCTL_AI_BUF_OVERFLOW		OS_IOCTL_RW(16,  4, s32)
#define	AISS16AO2_IOCTL_AI_BUF_UNDERFLOW	OS_IOCTL_RW(17,  4, s32)
#define	AISS16AO2_IOCTL_AI_BURST_ENABLE		OS_IOCTL_RW(18,  4, s32)
#define	AISS16AO2_IOCTL_AI_BURST_SIZE		OS_IOCTL_RW(19,  4, s32)
#define	AISS16AO2_IOCTL_AI_BURST_STATUS		OS_IOCTL_RW(20,  4, s32)
#define	AISS16AO2_IOCTL_AI_CHAN_SEL			OS_IOCTL_RW(21,  4, s32)
#define	AISS16AO2_IOCTL_AI_CLOCK_MODE		OS_IOCTL_RW(22,  4, s32)
#define	AISS16AO2_IOCTL_AI_ENABLE			OS_IOCTL_RW(23,  4, s32)
#define	AISS16AO2_IOCTL_AI_FSAMP_RANGE		OS_IOCTL_RW(24,  4, s32)
#define	AISS16AO2_IOCTL_AI_MODE				OS_IOCTL_RW(25,  4, s32)
#define	AISS16AO2_IOCTL_AI_RANGE_A			OS_IOCTL_RW(26,  4, s32)
#define	AISS16AO2_IOCTL_AI_RANGE_B			OS_IOCTL_RW(27,  4, s32)
#define	AISS16AO2_IOCTL_AI_SW_CLOCK			OS_IOCTL   (28)
#define	AISS16AO2_IOCTL_AI_SW_TRIGGER		OS_IOCTL   (29)
#define	AISS16AO2_IOCTL_AI_THRESH_LVL		OS_IOCTL_RW(30,  4, s32)
#define	AISS16AO2_IOCTL_AI_THRESH_STS		OS_IOCTL_RW(31,  4, s32)
#define	AISS16AO2_IOCTL_AO_BUF_CLEAR		OS_IOCTL   (32)
#define	AISS16AO2_IOCTL_AO_BUF_LEVEL		OS_IOCTL_RW(33,  4, s32)
#define	AISS16AO2_IOCTL_AO_BUF_LOAD_REQ		OS_IOCTL_R (34,  4, s32)
#define	AISS16AO2_IOCTL_AO_BUF_LOAD_STS		OS_IOCTL_R (35,  4, s32)
#define	AISS16AO2_IOCTL_AO_BUF_MODE			OS_IOCTL_RW(36,  4, s32)
#define	AISS16AO2_IOCTL_AO_BUF_OVER_DATA	OS_IOCTL_RW(37,  4, s32)
#define	AISS16AO2_IOCTL_AO_BUF_OVER_FRAM	OS_IOCTL_RW(38,  4, s32)
#define	AISS16AO2_IOCTL_AO_BURST_ENABLE		OS_IOCTL_RW(39,  4, s32)
#define	AISS16AO2_IOCTL_AO_BURST_STATUS		OS_IOCTL_R (40,  4, s32)
#define	AISS16AO2_IOCTL_AO_CHAN_SEL			OS_IOCTL_RW(41,  4, s32)
#define	AISS16AO2_IOCTL_AO_CLOCK_MODE		OS_IOCTL_RW(42,  4, s32)
#define	AISS16AO2_IOCTL_AO_CLOCK_STATUS		OS_IOCTL_R (43,  4, s32)
#define	AISS16AO2_IOCTL_AO_ENABLE			OS_IOCTL_RW(44,  4, s32)
#define	AISS16AO2_IOCTL_AO_MODE				OS_IOCTL_RW(45,  4, s32)
#define	AISS16AO2_IOCTL_AO_OUTPUT_MODE		OS_IOCTL_RW(46,  4, s32)
#define	AISS16AO2_IOCTL_AO_RANGE			OS_IOCTL_RW(47,  4, s32)
#define	AISS16AO2_IOCTL_AO_SW_CLOCK			OS_IOCTL   (48)
#define	AISS16AO2_IOCTL_AO_SW_TRIGGER		OS_IOCTL   (49)
#define	AISS16AO2_IOCTL_AO_THRESH_LVL		OS_IOCTL_RW(50,  4, s32)
#define	AISS16AO2_IOCTL_AO_THRESH_STS		OS_IOCTL_R (51,  4, s32)
#define	AISS16AO2_IOCTL_DATA_FORMAT			OS_IOCTL_RW(52,  4, s32)
#define	AISS16AO2_IOCTL_DIO_DIR_OUT			OS_IOCTL_RW(53,  4, s32)
#define	AISS16AO2_IOCTL_DIO_READ			OS_IOCTL_RW(54,  4, s32)
#define	AISS16AO2_IOCTL_DIO_WRITE			OS_IOCTL_RW(55,  4, s32)
#define	AISS16AO2_IOCTL_GEN_A_ENABLE		OS_IOCTL_RW(56,  4, s32)
#define	AISS16AO2_IOCTL_GEN_A_NDIV			OS_IOCTL_RW(57,  4, s32)
#define	AISS16AO2_IOCTL_GEN_B_ENABLE		OS_IOCTL_RW(58,  4, s32)
#define	AISS16AO2_IOCTL_GEN_B_NDIV			OS_IOCTL_RW(59,  4, s32)
#define	AISS16AO2_IOCTL_GEN_C_ENABLE		OS_IOCTL_RW(60,  4, s32)
#define	AISS16AO2_IOCTL_GEN_C_NDIV			OS_IOCTL_RW(61,  4, s32)
#define	AISS16AO2_IOCTL_IRQ_ENABLE			OS_IOCTL_RW(62,  4, s32)
#define	AISS16AO2_IOCTL_MASTER_CLOCK		OS_IOCTL_RW(63,  4, s32)
#define	AISS16AO2_IOCTL_TRIGGER_MODE		OS_IOCTL_RW(64,  4, s32)
#define	AISS16AO2_IOCTL_RX_IO_ABORT			OS_IOCTL_R (65,  4, s32)
#define	AISS16AO2_IOCTL_TX_IO_ABORT			OS_IOCTL_R (66,  4, s32)
#define	AISS16AO2_IOCTL_WAIT_EVENT			OS_IOCTL_RW(67, 28, gsc_wait_t)
#define	AISS16AO2_IOCTL_WAIT_CANCEL			OS_IOCTL_RW(68, 28, gsc_wait_t)
#define	AISS16AO2_IOCTL_WAIT_STATUS			OS_IOCTL_RW(69, 28, gsc_wait_t)


//*****************************************************************************
// AISS16AO2_IOCTL_REG_READ
// AISS16AO2_IOCTL_REG_WRITE
// AISS16AO2_IOCTL_REG_MOD
//
#define	AISS16AO2_REG_ENCODE(s,o)			GSC_REG_ENCODE(GSC_REG_TYPE_BAR2,(s),(o))
// Parameter:	gsc_reg_t*
#define	AISS16AO2_GSC_BCR					AISS16AO2_REG_ENCODE(4,0x00)// Board Control Register
#define	AISS16AO2_GSC_DIOPR					AISS16AO2_REG_ENCODE(4,0x04)// Digital I/O Port Register
#define	AISS16AO2_GSC_AOC0R					AISS16AO2_REG_ENCODE(4,0x08)// Anlog Output Channel 0 Register
#define	AISS16AO2_GSC_AOC1R					AISS16AO2_REG_ENCODE(4,0x0C)// Anlog Output Channel 1 Register

#define	AISS16AO2_GSC_AICR					AISS16AO2_REG_ENCODE(4,0x14)// Active Input Channels Register
#define	AISS16AO2_GSC_AIBR					AISS16AO2_REG_ENCODE(4,0x18)// Analog Input Buffer Register
#define	AISS16AO2_GSC_RGAR					AISS16AO2_REG_ENCODE(4,0x1C)// Rate Generator A Register
#define	AISS16AO2_GSC_RGBR					AISS16AO2_REG_ENCODE(4,0x20)// Rate Generator B Register
#define	AISS16AO2_GSC_AIBBSR				AISS16AO2_REG_ENCODE(4,0x24)// Analog Input Burst Block Size Register
#define	AISS16AO2_GSC_IBSR					AISS16AO2_REG_ENCODE(4,0x28)// Input Buffer Size Register
#define	AISS16AO2_GSC_IBTR					AISS16AO2_REG_ENCODE(4,0x2C)// Input Buffer Threshold Register
#define	AISS16AO2_GSC_PSR					AISS16AO2_REG_ENCODE(4,0x30)// Primary Status Register
#define	AISS16AO2_GSC_ACR					AISS16AO2_REG_ENCODE(4,0x34)// Assymbly Configuration Register
#define	AISS16AO2_GSC_AVR					AISS16AO2_REG_ENCODE(4,0x38)// Auto Cal Values Register
#define	AISS16AO2_GSC_BOOR					AISS16AO2_REG_ENCODE(4,0x3C)// Buffered Output Operations Register
#define	AISS16AO2_GSC_OBTR					AISS16AO2_REG_ENCODE(4,0x40)// Output Buffer Threshold Register
#define	AISS16AO2_GSC_OBSR					AISS16AO2_REG_ENCODE(4,0x44)// Output Buffer Size Register
#define	AISS16AO2_GSC_AOBR					AISS16AO2_REG_ENCODE(4,0x48)// Analog Output Buffer Register
#define	AISS16AO2_GSC_RGCR					AISS16AO2_REG_ENCODE(4,0x4C)// Rate Generator C Register

//*****************************************************************************
// AISS16AO2_IOCTL_QUERY
//
//	Parameter:	s32
//		Pass in a value from the list below.
//		The value returned is the answer to the query.

typedef enum
{
	AISS16AO2_QUERY_AUTO_CAL_MS,		// Max auto-cal period in ms.
	AISS16AO2_QUERY_CHANNEL_AI_MAX,		// Maximum number of input channels supported.
	AISS16AO2_QUERY_CHANNEL_AI_QTY,		// The number of input channels supported.
	AISS16AO2_QUERY_CHANNEL_AO_MAX,		// Maximum number of output channels supported.
	AISS16AO2_QUERY_CHANNEL_AO_QTY,		// The number of output channels supported.
	AISS16AO2_QUERY_COUNT,				// How many query options are supported?
	AISS16AO2_QUERY_DEVICE_TYPE,		// Value from gsc_dev_type_t
	AISS16AO2_QUERY_FGEN_MAX,			// Rate Generator maximum output rate.
	AISS16AO2_QUERY_FGEN_MIN,			// Rate Generator minimum output rate.
	AISS16AO2_QUERY_FIFO_SIZE_RX,		// Rx FIFO depth in 32-bit samples
	AISS16AO2_QUERY_FIFO_SIZE_TX,		// Tx FIFO depth in 32-bit samples
	AISS16AO2_QUERY_FSAMP_MAX,			// The maximum sample rate per channel.
	AISS16AO2_QUERY_FSAMP_MIN,			// The minimum sample rate per channel.
	AISS16AO2_QUERY_INIT_MS,			// Max initialize period in ms.
	AISS16AO2_QUERY_MASTER_CLOCK,		// Master clock frequency
	AISS16AO2_QUERY_NDIV_MASK,			// Mask of valid Ndiv bits.
	AISS16AO2_QUERY_NDIV_MAX,			// Maximum rate generator Ndiv value.
	AISS16AO2_QUERY_NDIV_MIN,			// Maximum rate generator Ndiv value.
	AISS16AO2_QUERY_RATE_GEN_QTY,		// Number of Rate Generatorts.
	AISS16AO2_QUERY_DMDMA				// Is DMDMA supported?

} aiss16ao2_query_t;

#define	AISS16AO2_IOCTL_QUERY_LAST			AISS16AO2_QUERY_DMDMA
#define	AISS16AO2_IOCTL_QUERY_ERROR			(-1)

//*****************************************************************************
// AISS16AO2_IOCTL_INITIALIZE
//
//	Parameter:	None

//*****************************************************************************
// AISS16AO2_IOCTL_AUTO_CALIBRATE
//
//	Parameter:	None

//*****************************************************************************
// AISS16AO2_IOCTL_AUTO_CAL_STS
//
//	Parameter:	s32*
//		One of the below values is returned.
#define	AISS16AO2_AUTO_CAL_STS_ACTIVE		0
#define	AISS16AO2_AUTO_CAL_STS_FAIL			1
#define	AISS16AO2_AUTO_CAL_STS_PASS			2

//*****************************************************************************
// AISS16AO2_IOCTL_RX_IO_MODE
// AISS16AO2_IOCTL_TX_IO_MODE
//
//	Parameter:	s32*
//		Pass in any of the gsc_io_mode_t options, or
//		-1 to read the current setting.
#define	AISS16AO2_IO_MODE_DEFAULT			GSC_IO_MODE_PIO
//*****************************************************************************
// AISS16AO2_IOCTL_RX_IO_OVERFLOW
// AISS16AO2_IOCTL_TX_IO_OVERFLOW
//
//	Parameter:	s32*
//		Pass in any of the below options, or
//		-1 to read the current setting.
#define	AISS16AO2_IO_OVERFLOW_DEFAULT		AISS16AO2_IO_OVERFLOW_CHECK
#define	AISS16AO2_IO_OVERFLOW_IGNORE		0
#define	AISS16AO2_IO_OVERFLOW_CHECK			1

//*****************************************************************************
// AISS16AO2_IOCTL_RX_IO_TIMEOUT (in seconds)
// AISS16AO2_IOCTL_TX_IO_TIMEOUT (in seconds)
//
//	Parameter:	s32*
//		Pass in any value from the minimim to the maximim, the infinite option,
//		or -1 to read the current setting. The value -1 is returned if the
//		feature is not supported, but this should never happen.
#define	AISS16AO2_IO_TIMEOUT_DEFAULT		10
#define	AISS16AO2_IO_TIMEOUT_NO_SLEEP		0
#define	AISS16AO2_IO_TIMEOUT_MIN			0
#define	AISS16AO2_IO_TIMEOUT_MAX			GSC_IO_TIMEOUT_MAX
#define	AISS16AO2_IO_TIMEOUT_INFINITE		GSC_IO_TIMEOUT_INFINITE

//*****************************************************************************
// AISS16AO2_IOCTL_RX_IO_UNDERFLOW
//
//	Parameter:	s32*
//		Pass in any of the below options, or
//		-1 to read the current setting.
#define	AISS16AO2_IO_UNDERFLOW_DEFAULT		AISS16AO2_IO_UNDERFLOW_CHECK
#define	AISS16AO2_IO_UNDERFLOW_IGNORE		0
#define	AISS16AO2_IO_UNDERFLOW_CHECK		1

//*****************************************************************************
// AISS16AO2_IOCTL_AI_BUF_CLEAR
// AISS16AO2_IOCTL_AO_BUF_CLEAR
//
//	Parameter:	None.

//*****************************************************************************
// AISS16AO2_IOCTL_AI_BUF_LEVEL
// AISS16AO2_IOCTL_AO_BUF_LEVEL
//
//	Parameter:	s32*
//		The value returned is the current buffer fill level.

//*****************************************************************************
// AISS16AO2_IOCTL_AI_BUF_OVERFLOW
// AISS16AO2_IOCTL_AO_BUF_OVER_DATA
// AISS16AO2_IOCTL_AO_BUF_OVER_FRAM
//
//	Parameter:	s32*
//		Pass in any valid option below, or
//		-1 to read the current setting.
#define	AISS16AO2_BUF_OVERFLOW_CLEAR		0
#define	AISS16AO2_BUF_OVERFLOW_TEST			(-1)

// For queries the following values are returned.
#define	AISS16AO2_BUF_OVERFLOW_NO			0
#define	AISS16AO2_BUF_OVERFLOW_YES			1

//*****************************************************************************
// AISS16AO2_IOCTL_AI_BUF_UNDERFLOW
//
//	Parameter:	s32*
//		Pass in any of the below options, or
//		-1 to read the current setting.
#define	AISS16AO2_BUF_UNDERFLOW_CLEAR		0
#define	AISS16AO2_BUF_UNDERFLOW_TEST		(-1)

// For queries the following values are returned.
#define	AISS16AO2_BUF_UNDERFLOW_NO			0
#define	AISS16AO2_BUF_UNDERFLOW_YES			1

//*****************************************************************************
// AISS16AO2_IOCTL_AI_BURST_ENABLE
// AISS16AO2_IOCTL_AO_BURST_ENABLE
//
//	Parameter:	s32*
//		The value returned is one of the following.
#define	AISS16AO2_BURST_ENABLE_NO			0
#define	AISS16AO2_BURST_ENABLE_YES			1

//*****************************************************************************
// AISS16AO2_IOCTL_AO_BURST_STATUS			BOOR D18
//
//	Parameter:	s32*
//		The value returned is one of the following.
#define	AISS16AO2_AO_BURST_STATUS_BUSY		0
#define	AISS16AO2_AO_BURST_STATUS_READY		1

//*****************************************************************************
// AISS16AO2_IOCTL_AI_BURST_SIZE
//
//	Parameter:	s32*
//		Pass in any value between the below inclusive limits, or
//		-1 to read the current setting.
//		At this time the valid range is 0-0xFFFFFF.

//*****************************************************************************
// AISS16AO2_IOCTL_AI_BURST_STATUS			BCR D10
//
//	Parameter:	s32*
//		The value returned is one of the following.
#define	AISS16AO2_AI_BURST_STATUS_READY		0
#define	AISS16AO2_AI_BURST_STATUS_BUSY		1

//*****************************************************************************
// AISS16AO2_IOCTL_AI_CHAN_SEL
// AISS16AO2_IOCTL_AO_CHAN_SEL
//
//	Parameter:	s32*
//		Pass in a bitmap of the channels to enable, or
//		-1 to read the current setting.
//		A valid bitmap is dependent on the AI or AO selection and the
//		number of corresponding input or output channels on the device.

//*****************************************************************************
// AISS16AO2_IOCTL_AI_CLOCK_MODE
// AISS16AO2_IOCTL_AO_CLOCK_MODE
//
//	Parameter:	s32*
//		Pass in any of the below options, or
//		-1 to read the current setting.
#define	AISS16AO2_CLOCK_MODE_IN				0
#define	AISS16AO2_CLOCK_MODE_OUT			1

//*****************************************************************************
// AISS16AO2_IOCTL_AI_ENABLE
// AISS16AO2_IOCTL_AO_ENABLE
//
//	Parameter:	s32*
//		Pass in any valid option below, or
//		-1 to read the current setting.
#define	AISS16AO2_AIO_ENABLE_NO				0	// Disable the buffer.
#define	AISS16AO2_AIO_ENABLE_YES			1	// Enable the buffer.

//*****************************************************************************
// AISS16AO2_IOCTL_AI_FSAMP_RANGE
//
//	Parameter:	s32*
//		Pass in any of the below options, or
//		-1 to read the current setting.
#define	AISS16AO2_FSAMP_RANGE_LOW			0
#define	AISS16AO2_FSAMP_RANGE_HIGH			1

//*****************************************************************************
// AISS16AO2_IOCTL_AI_MODE
//
//	Parameter:	s32*
//		Pass in any of the below options, or
//		-1 to read the current setting.
#define	AISS16AO2_AI_MODE_DIFF				0	// Differential
#define	AISS16AO2_AI_MODE_SINGLE			1	// Single-Ended
#define	AISS16AO2_AI_MODE_ZERO				2	// Zero test
#define	AISS16AO2_AI_MODE_VREF				3	// Vref test
#define	AISS16AO2_AI_MODE_AO_0				4	// Analog Output Channel 0
#define	AISS16AO2_AI_MODE_AO_1				5	// Analog Output Channel 1

//*****************************************************************************
// AISS16AO2_IOCTL_AI_RANGE_A
// AISS16AO2_IOCTL_AI_RANGE_B
// AISS16AO2_IOCTL_AO_RANGE
//
//	Parameter:	s32*
//		Pass in any of the below options, or
//		-1 to read the current setting.
#define	AISS16AO2_RANGE_2_5V				0	// +- 2.5 volts
#define	AISS16AO2_RANGE_5V					1	// +- 5 volts
#define	AISS16AO2_RANGE_10V					2	// +- 10 volts

//*****************************************************************************
// AISS16AO2_IOCTL_AI_SW_CLOCK
// AISS16AO2_IOCTL_AO_SW_CLOCK
//
//	Parameter:	None.

//*****************************************************************************
// AISS16AO2_IOCTL_AI_SW_TRIGGER
// AISS16AO2_IOCTL_AO_SW_TRIGGER
//
//	Parameter:	None.

//*****************************************************************************
// AISS16AO2_IOCTL_AI_THRESH_LVL
// AISS16AO2_IOCTL_AO_THRESH_LVL
//
//	Parameter:	s32*
//		Pass in any valid value from 0x0 to 0x7FFFF, or
//		-1 to read the current setting.

//*****************************************************************************
// AISS16AO2_IOCTL_AI_THRESH_STS
// AISS16AO2_IOCTL_AO_THRESH_STS
//
//	Parameter:	s32*
//		The value returned is one of the following.
#define	AISS16AO2_THRESH_STS_CLEAR			0
#define	AISS16AO2_THRESH_STS_SET			1

//*****************************************************************************
// AISS16AO2_IOCTL_AO_BUF_LOAD_REQ
//
//	Parameter:	None.

//*****************************************************************************
// AISS16AO2_IOCTL_AO_BUF_LOAD_STS
//
//	Parameter:	s32*
//		The value returned is one of the following.
#define	AISS16AO2_BUF_LOAD_STS_NOT_READY	0
#define	AISS16AO2_BUF_LOAD_STS_READY		1

//*****************************************************************************
// AISS16AO2_IOCTL_AO_BUF_MODE
//
//	Parameter:	s32*
//		Pass in any of the below options, or
//		-1 to read the current setting.
#define	AISS16AO2_BUF_MODE_OPEN				0
#define	AISS16AO2_BUF_MODE_CIRC				1

//*****************************************************************************
// AISS16AO2_IOCTL_AO_CLOCK_STATUS
//
//	Parameter:	s32*
//		The value returned is one of the following.
#define	AISS16AO2_CLOCK_STATUS_NOT_READY	0
#define	AISS16AO2_CLOCK_STATUS_READY		1

//*****************************************************************************
// AISS16AO2_IOCTL_AO_MODE
//
//	Parameter:	s32*
//		Pass in any of the below options, or
//		-1 to read the current setting.
#define	AISS16AO2_AO_MODE_REG				0
#define	AISS16AO2_AO_MODE_FIFO				1

//*****************************************************************************
// AISS16AO2_IOCTL_AO_OUTPUT_MODE
//
//	Parameter:	s32*
//		Pass in any of the below options, or
//		-1 to read the current setting.
#define	AISS16AO2_AO_OUTPUT_MODE_IMMED		0
#define	AISS16AO2_AO_OUTPUT_MODE_SIMUL		1

//*****************************************************************************
// AISS16AO2_IOCTL_DATA_FORMAT
//
//	Parameter:	s32*
//		Pass in any of the below options, or
//		-1 to read the current setting.
#define	AISS16AO2_DATA_FORMAT_2S_COMP		0	// Twos Compliment
#define	AISS16AO2_DATA_FORMAT_OFF_BIN		1	// Offset Binary

//*****************************************************************************
// AISS16AO2_IOCTL_DIO_DIR_OUT
//
//	Parameter:	s32*
//		Pass in any value between the range of 0 to 0x3F, or
//		-1 to read the current setting.

//*****************************************************************************
// AISS16AO2_IOCTL_DIO_READ
//
//	Parameter:	s32*
//		The value returned is in the range of 0 to 0x3F.

//*****************************************************************************
// AISS16AO2_IOCTL_DIO_WRITE
//
//	Parameter:	s32*
//		Pass in any value in the range of 0 to 0x3F, or
//		-1 to read the current setting.

//*****************************************************************************
// AISS16AO2_IOCTL_GEN_A_ENABLE
// AISS16AO2_IOCTL_GEN_B_ENABLE
// AISS16AO2_IOCTL_GEN_C_ENABLE
//
//	Parameter:	s32*
//		Pass in any of the below options, or
//		-1 to read the current setting.
#define	AISS16AO2_GEN_ENABLE_NO				0
#define	AISS16AO2_GEN_ENABLE_YES			1

//*****************************************************************************
// AISS16AO2_IOCTL_GEN_A_NDIV
// AISS16AO2_IOCTL_GEN_B_NDIV
// AISS16AO2_IOCTL_GEN_C_NDIV
//
//	Parameter:	s32*
//		Pass in any value in the range of 45 to 0xFFFFFF, or
//		-1 to read the current setting.

//*****************************************************************************
// AISS16AO2_IOCTL_IRQ_ENABLE
//
//	Parameter:	s32*
//		Pass in any combination of the below options, or
//		-1 to read the current setting.
#define	AISS16AO2_IRQ_AUTO_CAL_DONE			0x0001
#define	AISS16AO2_IRQ_AI_THRESH_H2L			0x0002
#define	AISS16AO2_IRQ_AI_THRESH_L2H			0x0004
#define	AISS16AO2_IRQ_AI_FAULT				0x0008
#define	AISS16AO2_IRQ_AI_BURST_START		0x0010
#define	AISS16AO2_IRQ_AI_BURST_DONE			0x0020
#define	AISS16AO2_IRQ_AI_CLOCK				0x0040
#define	AISS16AO2_IRQ_AO_CLOCK				0x0080
#define	AISS16AO2_IRQ_DIO_0_L2H				0x0100
#define	AISS16AO2_IRQ_AO_THRESH_H2L			0x0200
#define	AISS16AO2_IRQ_AO_THRESH_L2H			0x0400
#define	AISS16AO2_IRQ_AO_LOAD_RDY_H2L		0x0800
#define	AISS16AO2_IRQ_AO_LOAD_RDY_L2H		0x1000
#define	AISS16AO2_IRQ_AO_BURST_READY		0x2000
#define	AISS16AO2_IRQ_AO_FAULT				0x4000
#define	AISS16AO2_IRQ_ALL					0x7FFF

//*****************************************************************************
// AISS16AO2_IOCTL_MASTER_CLOCK
//
//	Parameter:	s32*
//		Pass in any value in the below inclusive range, or
//		-1 to read the current setting.
#define	AISS16AO2_MASTER_CLOCK_MIN			30000000L
#define	AISS16AO2_MASTER_CLOCK_MAX			47000000L

//*****************************************************************************
// AISS16AO2_IOCTL_TRIGGER_MODE
//
//	Parameter:	s32*
//		Pass in any of the below options, or
//		-1 to read the current setting.
#define	AISS16AO2_TRIGGER_MODE_IN			0
#define	AISS16AO2_TRIGGER_MODE_OUT			1

//*****************************************************************************
// AISS16AO2_IOCTL_RX_IO_ABORT
// AISS16AO2_IOCTL_TX_IO_ABORT
//
//	Parameter:	s32*
//		The returned value is one of the below options.
#define	AISS16AO2_IO_ABORT_NO				0
#define	AISS16AO2_IO_ABORT_YES				1

//*****************************************************************************
// AISS16AO2_IOCTL_WAIT_EVENT				all fields must be valid
// AISS16AO2_IOCTL_WAIT_CANCEL				fields need not be valid
// AISS16AO2_IOCTL_WAIT_STATUS				fields need not be valid
//
//	Parameter:	gsc_wait_t*
// gsc_wait_t.flags - see gsc_common.h
// gsc_wait_t.main - see gsc_common.h
// gsc_wait_t.gsc
#define	AISS16AO2_WAIT_GSC_AUTO_CAL_DONE	AISS16AO2_IRQ_AUTO_CAL_DONE
#define	AISS16AO2_WAIT_GSC_AI_THRESH_H2L	AISS16AO2_IRQ_AI_THRESH_H2L
#define	AISS16AO2_WAIT_GSC_AI_THRESH_L2H	AISS16AO2_IRQ_AI_THRESH_L2H
#define	AISS16AO2_WAIT_GSC_AI_FAULT			AISS16AO2_IRQ_AI_FAULT
#define	AISS16AO2_WAIT_GSC_AI_BURST_STRT	AISS16AO2_IRQ_AI_BURST_START
#define	AISS16AO2_WAIT_GSC_AI_BURST_DONE	AISS16AO2_IRQ_AI_BURST_DONE
#define	AISS16AO2_WAIT_GSC_AI_CLOCK			AISS16AO2_IRQ_AI_CLOCK
#define	AISS16AO2_WAIT_GSC_AO_CLOCK			AISS16AO2_IRQ_AO_CLOCK
#define	AISS16AO2_WAIT_GSC_DIO_0_L2H		AISS16AO2_IRQ_DIO_0_L2H
#define	AISS16AO2_WAIT_GSC_AO_THRESH_H2L	AISS16AO2_IRQ_AO_THRESH_H2L
#define	AISS16AO2_WAIT_GSC_AO_THRESH_L2H	AISS16AO2_IRQ_AO_THRESH_L2H
#define	AISS16AO2_WAIT_GSC_AO_LOAD_RDY_L	AISS16AO2_IRQ_AO_LOAD_RDY_H2L
#define	AISS16AO2_WAIT_GSC_AO_LOAD_RDY_H	AISS16AO2_IRQ_AO_LOAD_RDY_L2H
#define	AISS16AO2_WAIT_GSC_AO_BURST_RDY		AISS16AO2_IRQ_AO_BURST_READY
#define	AISS16AO2_WAIT_GSC_AO_FAULT			AISS16AO2_IRQ_AO_FAULT
#define	AISS16AO2_WAIT_GSC_ALL				AISS16AO2_IRQ_ALL
// gsc_wait_t.alt flags
#define	AISS16AO2_WAIT_ALT_ALL				0x0000
// gsc_wait_t.io
#define	AISS16AO2_WAIT_IO_RX_ABORT			0x0001
#define	AISS16AO2_WAIT_IO_RX_DONE			0x0002
#define	AISS16AO2_WAIT_IO_RX_ERROR			0x0004
#define	AISS16AO2_WAIT_IO_RX_TIMEOUT		0x0008
#define	AISS16AO2_WAIT_IO_TX_ABORT			0x0010
#define	AISS16AO2_WAIT_IO_TX_DONE			0x0020
#define	AISS16AO2_WAIT_IO_TX_ERROR			0x0040
#define	AISS16AO2_WAIT_IO_TX_TIMEOUT		0x0080
#define	AISS16AO2_WAIT_IO_ALL				0x00FF



#endif
