// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AISS16AO2/driver/main.h $
// $Rev: 52935 $
// $Date: 2023-04-20 15:51:33 -0500 (Thu, 20 Apr 2023) $

// 16AISS16AO2: Device Driver: header file

#ifndef	__MAIN_H__
#define	__MAIN_H__

#include "16aiss16ao2.h"

#define	DEV_BAR_SHOW						0
#define	DEV_METRICS_SHOW					0
#define	DEV_PCI_ID_SHOW						0
#define	DEV_SUPPORTS_IRQ
#define	DEV_SUPPORTS_READ
#define	DEV_SUPPORTS_WAIT
#define	DEV_SUPPORTS_WRITE
#define	GSC_ALT_DEV_GET(a)					(a)
#define	GSC_ALT_STRUCT_GET(i,d)				(d)
#define	GSC_ALT_STRUCT_T					dev_data_t
#define	GSC_DEVS_PER_BOARD					1

typedef struct _dev_data_t					dev_data_t;

#include "gsc_main.h"



// macros *********************************************************************

#define	DEV_MODEL							"16AISS16AO2"	// Upper case form of the below.
#define	DEV_NAME							"16aiss16ao2"	// MUST AGREE WITH AISS16AO2_BASE_NAME

#define	DEV_VERSION							"2.5"			// FOR DEVICE SPECIFIC CODE ONLY!
// 2.5	Updated to support the 6.x series kernel.
//		Updated comments for consistency.
//		Updated the Initialize IOCTL service for consistency.
//		Updated the Autocalibration IOCTL service for consistency.
//		Updated the AI and AO buffer clear IOCTL services for consistency.
//		Updated per PLX code segregation.
// 2.4	Corrected code in which open succeeds even if initilization fails.
//		Standardized various IOCTL system log messages.
//		Standardized processing of the initialization IOCTL service.
//		Standardized processing of the autocalibration IOCTL service.
//		Updated comments for consistency.
//		Changed the AI/AO Buffer Over/Underflow IGNORE options to TEST for clarity.
// 2.3	Added support for the 5.x kernel series.
//		White space cleanup.
//		Redefined a macro text, though not its value (...WAIT_IO_ALL)
//		Updated some comments.
//		Removed some redundant code.
// 2.2	Bug fix: was ignoring BAR setup return value.
//		Updated per changes to the OS specific PCI services.
//		Made updates for DEV_SUPPORTS_XXX macro changes.
//		Modified code so open succeeds even if initialization fails.
//		Updated per changes to the common I/O code.
//		Updated the Block Mode DMA available services to use the PIO available service.
// 2.1	The BAR code has been updated to include the data under a single structure.
//		The register definitions have been updated.
//		I/O services now pass around an os_mem_t structure instead of a buffer pointer.
//		White space cleanup.
//		Changed the arbitrary wait event callback argument to a void* type.
//		Modified to allow multiple apps to simultaneously access a single device.
//		Correct access to BAR0 and BAR1 (is RO, was RW).
//		Added an infinite I/O timeout option.
//		Corrected the IOCTL code for the register write and mod services.
// 2.0	Updated to use the newer common driver sources.
//		Removed GNU notice from non-Linux specific files.
//		Removed Linux specific content from non-Linux specific source files.
//		White space cleanup.
//		Now using a spinlock rather than enabling and disabling interrupts.
//		Updated gsc_irq_open() and gsc_irq_close().
//		Updated gsc_dma_open() and gsc_dma_close().
// 1.4	Reduced #include list in driver interface header.
// 1.3	Updated for the 3.x kernel.
//		Corrected a system time tick count roll over bug in ioctl.c.
// 1.2	Updated per changes to the common source code.
//		Implemented IOCTL based I/O Abort support.
//		Implemented IOCTL based event wait support.
//		Updated to use the common gsc_open() and gsc_close() services.
//		Implemented support for the common PIO I/O routines.
//		Changed use of DEV_SUPPORTS_READ macro.
//		Changed use of DEV_SUPPORTS_WRITE macro.
//		Eliminated the global dev_check_id() routine.
//		Removed compile warnings for Fedora 15.
//		Added AISS16AO2_QUERY_DMDMA as a query option.
//		Updated the AI/AO_BURST_STATUS value option macro names - bug fix.
// 1.1	Fixed bug in the AISS16AO2_IOCTL_AI_THRESH_STS service.
//		Modified to use the common open and close routines.
//		Removed unused macros.
// 1.0	Initial release.
// BETA.0	Preliminary release of the new driver.

// I/O services
#define	DEV_IO_STREAM_QTY					(DEV_IO_RX_STREAM_QTY + DEV_IO_TX_STREAM_QTY)
#define	DEV_IO_RX_STREAM_QTY				1
#define	DEV_IO_TX_STREAM_QTY				2

#define	GSC_READ_PIO_WORK_32_BIT
#define	GSC_WRITE_PIO_WORK_32_BIT

// WAIT services
#define	DEV_WAIT_GSC_ALL					AISS16AO2_WAIT_GSC_ALL
#define	DEV_WAIT_ALT_ALL					AISS16AO2_WAIT_ALT_ALL
#define	DEV_WAIT_IO_ALL						AISS16AO2_WAIT_IO_ALL



// data types *****************************************************************

struct _dev_io_t
{
	// Initialized by open and IOCTL services (initialize and/or service specific).

	s32					io_mode;			// PIO, BMDMA, DMDMA
	s32					overflow_check;		// Check for overflows?
	s32					pio_threshold;		// Use PIO if samples <= this.
	s32					timeout_s;			// I/O timeout in seconds.
	s32					underflow_check;	// Check for underflows?

	// Initialized by I/O service at time of use.

	int					abort;				// Also set via IOCTL service.
	gsc_dma_ch_t*		dma_channel;		// Use this channel for DMA.
	int					non_blocking;		// Is this non-blocking I/O?

	// Initialized by device specific I/O create code.

	int					bytes_per_sample;	// Sample size in bytes.
	u32					io_reg_offset;		// Offset of device's I/O FIFO.
	VADDR_T				io_reg_vaddr;		// Address of device's I/O FIFO.

	void				(*dev_io_sw_init)		(GSC_ALT_STRUCT_T* alt, dev_io_t* io);
	void				(*dev_io_close)			(GSC_ALT_STRUCT_T* alt, dev_io_t* io);
	void				(*dev_io_open)			(GSC_ALT_STRUCT_T* alt, dev_io_t* io);
	int					(*dev_io_startup)		(GSC_ALT_STRUCT_T* alt, dev_io_t* io);
	long				(*dev_pio_available)	(GSC_ALT_STRUCT_T* alt, dev_io_t* io, size_t count);
	long				(*dev_bmdma_available)	(GSC_ALT_STRUCT_T* alt, dev_io_t* io, size_t count);
	long				(*dev_dmdma_available)	(GSC_ALT_STRUCT_T* alt, dev_io_t* io, size_t count);
	long				(*dev_pio_xfer)			(GSC_ALT_STRUCT_T* alt, dev_io_t* io, const os_mem_t* mem, size_t count, os_time_tick_t st_end);
	long				(*dev_bmdma_xfer)		(GSC_ALT_STRUCT_T* alt, dev_io_t* io, const os_mem_t* mem, size_t count, os_time_tick_t st_end);
	long				(*dev_dmdma_xfer)		(GSC_ALT_STRUCT_T* alt, dev_io_t* io, const os_mem_t* mem, size_t count, os_time_tick_t st_end);

	struct
	{
		u32				abort;		// ...WAIT_IO_XXX_ABORT
		u32				done;		// ...WAIT_IO_XXX_DONE
		u32				error;		// ...WAIT_IO_XXX_ERROR
		u32				timeout;	// ...WAIT_IO_XXX_TIMEOUT
	} wait;

	// Initialize by GSC commone create code.

	os_mem_t			mem;				// I/O buffer.
	os_sem_t			sem;				// Only one Tx or Rx at a time.
};

struct _dev_data_t
{
	os_pci_t			pci;			// The kernel PCI device descriptor.
	os_data_t			os;				// OS specific data.
	os_spinlock_t		spinlock;		// Control ISR access.
	os_sem_t			sem;			// Control thread access.
	gsc_dev_type_t		board_type;		// Corresponds to basic device type.
	const char*			model;			// Base model number in upper case.
	int					board_index;	// Index of the device being accessed.
	int					users;			// Number of currently active open() requests.
	int					share;			// Were we opened in shared mode?

	gsc_bar_t			bar;			// device register mappings
	gsc_dma_t			dma;			// For DMA based I/O.
	gsc_irq_t			irq;			// For interrut support.
	gsc_wait_node_t*	wait_list;

	struct
	{					// This is for streaming I/O
		dev_io_t		rx;				// For read operations.
		dev_io_t		tx;				// For write operations.
		dev_io_t*		io_streams[DEV_IO_STREAM_QTY];
	} io;

	struct
	{
		VADDR_T			plx_intcsr_32;	// Interrupt Control/Status Register
		VADDR_T			plx_dmaarb_32;	// DMA Arbitration Register
		VADDR_T			plx_dmathr_32;	// DMA Threshold Register

		VADDR_T			gsc_bcr_32;		// 0x00 Board Control Register
		VADDR_T			gsc_diopr_32;	// 0x04 Digital I/O Port Register

		VADDR_T			gsc_aicr_32;	// 0x14 Active Input Channels Register
		VADDR_T			gsc_aibr_32;	// 0x18 Analog Input Buffer Register
		VADDR_T			gsc_rgar_32;	// 0x1C Rate Generator A Register
		VADDR_T			gsc_rgbr_32;	// 0x20 Rate Generator B Register
		VADDR_T			gsc_aibbsr_32;	// 0x24 Analog Input Burst Block Size Register
		VADDR_T			gsc_ibsr_32;	// 0x28 Input Buffer Size Register
		VADDR_T			gsc_ibtr_32;	// 0x2C Input Buffer Threshold Register
		VADDR_T			gsc_psr_32;		// 0x30 Primary Status Register
		VADDR_T			gsc_acr_32;		// 0x34 Assembly Configuration Register

		VADDR_T			gsc_boor_32;	// 0x3C Buffered Output Operations Register
		VADDR_T			gsc_obtr_32;	// 0x40 Output Buffer Threshold Register
		VADDR_T			gsc_obsr_32;	// 0x44 Ouput Buffer Size Register
		VADDR_T			gsc_aobr_32;	// 0x48 Analog Output Buffer Register
		VADDR_T			gsc_rgcr_32;	// 0x4C Rate Generator C Register

	} vaddr;

	struct
	{
		s32				ai_chans_max;		// Maximum input channels supported.
		s32				ai_chan_qty;		// The number of input channels on the device.
		s32				ao_chans_max;		// Maximum output channels supported.
		s32				ao_chan_qty;		// The number of output channels on the device.
		s32				auto_cal_ms;		// Maximum ms for auto-cal

		s32				dmdma;				// Is DMDMA supported?

		u32				fifo_size_rx;		// Size of Rx FIFO - not the fill level.
		u32				fifo_size_tx;		// Size of Tx FIFO - not the fill level.
		s32				fsamp_max;			// The maximum Fsamp rate per channel.
		s32				fsamp_min;			// The minimum Fsamp rate per channel.

		u32				gsc_acr_32;			// Assembly Configuration Register

		s32				initialize_ms;		// Maximum ms for initialize

		s32				master_clock;		// Master clock frequency

		s32				rate_gen_fgen_max;	// Rate Generator maximum output rate.
		s32				rate_gen_fgen_min;	// Rate Generator minimum output rate.
		s32				rate_gen_ndiv_mask;	// Mask of valid Ndiv bits.
		s32				rate_gen_ndiv_max;	// Minimum Ndiv value.
		s32				rate_gen_ndiv_min;	// Maximum Ndiv value.
		s32				rate_gen_qty;		// The number of rate generators on device.

	} cache;
};



// prototypes *****************************************************************

int			dev_read_create(dev_data_t* dev, dev_io_t* io);
int			dev_write_create(dev_data_t* dev, dev_io_t* io);
void		dev_io_close(dev_data_t* dev);
int			dev_io_create(dev_data_t* dev);
void		dev_io_destroy(dev_data_t* dev);
int			dev_io_open(dev_data_t* dev);
dev_io_t*	dev_io_read_select(dev_data_t* dev, size_t count);
dev_io_t*	dev_io_write_select(dev_data_t* dev, size_t count);
int			dev_irq_create(dev_data_t* dev);
void		dev_irq_destroy(dev_data_t* dev);

int			initialize_ioctl(dev_data_t* dev, void* arg);



#endif
