// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AISS16AO2/samples/sbtest/ai_mode.c $
// $Rev: 52930 $
// $Date: 2023-04-20 15:46:28 -0500 (Thu, 20 Apr 2023) $

// 16AISS16AO2: Sample Application: source file

#include "main.h"



// macros *********************************************************************

#define	OUT0		0xAA3D			// +3.3 volts
#define	OUT1		0xD47A			// +6.6 volts

#define	OUT0_MAX	(OUT0 + 328)	// +3.3 +-0.1 volts
#define	OUT0_MIN	(OUT0 - 328)
#define	OUT1_MAX	(OUT1 + 328)	// +6.6 +-0.1 volts
#define	OUT1_MIN	(OUT1 - 328)
#define	VREF_MAX	(0xFEB8 + 328)	// +9.9 +-0.1 volts
#define	VREF_MIN	(0xFEB8 - 328)
#define	ZERO_MAX	(0x8000 + 328)	// 0.0 +-0.1 volts
#define	ZERO_MIN	(0x8000 - 328)



//*****************************************************************************
static int _service_test(int fd)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_AI_MODE,
			/* arg		*/	AISS16AO2_AI_MODE_DIFF,
			/* reg		*/	AISS16AO2_GSC_BCR,
			/* mask		*/	0x7,
			/* value	*/	0x0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_AI_MODE,
			/* arg		*/	AISS16AO2_AI_MODE_SINGLE,
			/* reg		*/	AISS16AO2_GSC_BCR,
			/* mask		*/	0x7,
			/* value	*/	0x1
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_AI_MODE,
			/* arg		*/	AISS16AO2_AI_MODE_ZERO,
			/* reg		*/	AISS16AO2_GSC_BCR,
			/* mask		*/	0x7,
			/* value	*/	0x2
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_AI_MODE,
			/* arg		*/	AISS16AO2_AI_MODE_VREF,
			/* reg		*/	AISS16AO2_GSC_BCR,
			/* mask		*/	0x7,
			/* value	*/	0x3
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_AI_MODE,
			/* arg		*/	AISS16AO2_AI_MODE_AO_0,
			/* reg		*/	AISS16AO2_GSC_BCR,
			/* mask		*/	0x7,
			/* value	*/	0x4
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_AI_MODE,
			/* arg		*/	AISS16AO2_AI_MODE_AO_1,
			/* reg		*/	AISS16AO2_GSC_BCR,
			/* mask		*/	0x7,
			/* value	*/	0x5
		},

		{ SERVICE_END_LIST }
	};

	int errs	= 0;

	errs	+= aiss16ao2_initialize(fd, -1, 0);

	errs	+= service_ioctl_set_reg_list(fd, list);
	errs	+= service_ioctl_reg_get_list(fd, list);

	errs	+= service_ioctl_set_reg_list(fd, list);
	errs	+= service_ioctl_reg_get_list(fd, list);

	errs	+= aiss16ao2_initialize(fd, -1, 0);
	return(errs);
}




//*****************************************************************************
static int _function_test(int fd)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_DATA_FORMAT,
			/* arg		*/	AISS16AO2_DATA_FORMAT_OFF_BIN,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_AI_CLOCK_MODE,
			/* arg		*/	AISS16AO2_CLOCK_MODE_OUT,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_TRIGGER_MODE,
			/* arg		*/	AISS16AO2_TRIGGER_MODE_OUT,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		// Configure the outputs.

		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_AO_RANGE,
			/* arg		*/	AISS16AO2_RANGE_10V,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_AO_MODE,
			/* arg		*/	AISS16AO2_AO_MODE_REG,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_AO_OUTPUT_MODE,
			/* arg		*/	AISS16AO2_AO_OUTPUT_MODE_IMMED,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		// Configure the inputs.

		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_AI_RANGE_A,
			/* arg		*/	AISS16AO2_RANGE_10V,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_AI_RANGE_B,
			/* arg		*/	AISS16AO2_RANGE_10V,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_GEN_A_NDIV,
			/* arg		*/	90,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_GEN_A_ENABLE,
			/* arg		*/	AISS16AO2_GEN_ENABLE_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_AI_FSAMP_RANGE,
			/* arg		*/	AISS16AO2_FSAMP_RANGE_LOW,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_AI_CHAN_SEL,
			/* arg		*/	0x1,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	100,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_AI_ENABLE,
			/* arg		*/	AISS16AO2_AIO_ENABLE_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	10,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	static const service_data_t	out0[]	=
	{
		// Verify the Output Channel 0 option.

		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_AI_MODE,
			/* arg		*/	AISS16AO2_AI_MODE_AO_0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_AUTO_CALIBRATE,
			/* arg		*/	0,
			/* reg		*/	AISS16AO2_GSC_BCR,
			/* mask		*/	0x30000000,
			/* value	*/	0x20000000
		},
		{
			/* service	*/	SERVICE_REG_MOD,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	AISS16AO2_GSC_AOC0R,
			/* mask		*/	0xFFFF,
			/* value	*/	OUT0
		},
		{
			/* service	*/	SERVICE_REG_MOD,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	AISS16AO2_GSC_AOC0R,
			/* mask		*/	0xFFFF,
			/* value	*/	OUT0
		},
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	100,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_AI_BUF_CLEAR,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	10,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	static const service_data_t	out1[]	=
	{
		// Verify the Output Channel 1 option.

		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_AI_MODE,
			/* arg		*/	AISS16AO2_AI_MODE_AO_1,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_AUTO_CALIBRATE,
			/* arg		*/	0,
			/* reg		*/	AISS16AO2_GSC_BCR,
			/* mask		*/	0x30000000,
			/* value	*/	0x20000000
		},
		{
			/* service	*/	SERVICE_REG_MOD,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	AISS16AO2_GSC_AOC1R,
			/* mask		*/	0xFFFF,
			/* value	*/	OUT1
		},
		{
			/* service	*/	SERVICE_REG_MOD,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	AISS16AO2_GSC_AOC1R,
			/* mask		*/	0xFFFF,
			/* value	*/	OUT1
		},
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	100,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_AI_BUF_CLEAR,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	10,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	static const service_data_t	vref[]	=
	{
		// Verify the Vref Test option.

		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_AI_MODE,
			/* arg		*/	AISS16AO2_AI_MODE_VREF,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_AUTO_CALIBRATE,
			/* arg		*/	0,
			/* reg		*/	AISS16AO2_GSC_BCR,
			/* mask		*/	0x30000000,
			/* value	*/	0x20000000
		},
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	100,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_AI_BUF_CLEAR,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	10,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	static const service_data_t	zero[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_AI_MODE,
			/* arg		*/	AISS16AO2_AI_MODE_ZERO,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_AUTO_CALIBRATE,
			/* arg		*/	0,
			/* reg		*/	AISS16AO2_GSC_BCR,
			/* mask		*/	0x30000000,
			/* value	*/	0x20000000
		},
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	100,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_AI_BUF_CLEAR,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	10,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	int			errs	= 0;
	gsc_reg_t	parm;
	int			ret;
	s32			value;

	errs	+= aiss16ao2_initialize(fd, -1, 0);

	errs	+= service_ioctl_set_reg_list(fd, list);

	// Verify the Output Channel 0  Test option.
	errs		+= service_ioctl_set_reg_list(fd, out0);
	parm.reg	= AISS16AO2_GSC_AIBR;
	parm.value	= 0xDEADBEEF;
	parm.mask	= 0xBEEFDEAD;
	ret			= aiss16ao2_ioctl(fd, AISS16AO2_IOCTL_REG_READ, (void*) &parm);
	value		= parm.value & 0xFFFF;

	if (ret < 0)
	{
		errs++;
		printf(	"FAIL <---  (%d. aiss16ao2_ioctl() failure, error %d)\n",
				__LINE__,
				ret);
	}
	else if ((value < OUT0_MIN) || (value > OUT0_MAX))
	{
		errs++;
		printf(	"FAIL <---  (%d. Output Channel 0 error: 0x%04lX)\n",
				__LINE__,
				(long) value);
	}

	// Verify the Output Channel 1  Test option.
	errs		+= service_ioctl_set_reg_list(fd, out1);
	parm.reg	= AISS16AO2_GSC_AIBR;
	parm.value	= 0xDEADBEEF;
	parm.mask	= 0xBEEFDEAD;
	ret			= aiss16ao2_ioctl(fd, AISS16AO2_IOCTL_REG_READ, (void*) &parm);
	value		= parm.value & 0xFFFF;

	if (ret < 0)
	{
		errs++;
		printf(	"FAIL <---  (%d. aiss16ao2_ioctl() failure, error %d)\n",
				__LINE__,
				ret);
	}
	else if ((value < OUT1_MIN) || (value > OUT1_MAX))
	{
		errs++;
		printf(	"FAIL <---  (%d. Output Channel 1 error: 0x%04lX)\n",
				__LINE__,
				(long) value);
	}

	// Verify the Zero Test option.
	errs		+= service_ioctl_set_reg_list(fd, zero);
	parm.reg	= AISS16AO2_GSC_AIBR;
	parm.value	= 0xDEADBEEF;
	parm.mask	= 0xBEEFDEAD;
	ret			= aiss16ao2_ioctl(fd, AISS16AO2_IOCTL_REG_READ, (void*) &parm);
	value		= parm.value & 0xFFFF;

	if (ret < 0)
	{
		errs++;
		printf(	"FAIL <---  (%d. aiss16ao2_ioctl() failure, error %d)\n",
				__LINE__,
				ret);
	}
	else if ((value < ZERO_MIN) || (value > ZERO_MAX))
	{
		errs++;
		printf(	"FAIL <---  (%d. Zero error: 0x%04lX)\n",
				__LINE__,
				(long) value);
	}

	// Verify the Vref Test option.
	errs		+= service_ioctl_set_reg_list(fd, vref);
	parm.reg	= AISS16AO2_GSC_AIBR;
	parm.value	= 0xDEADBEEF;
	parm.mask	= 0xBEEFDEAD;
	ret			= aiss16ao2_ioctl(fd, AISS16AO2_IOCTL_REG_READ, (void*) &parm);
	value		= parm.value & 0xFFFF;

	if (ret < 0)
	{
		errs++;
		printf(	"FAIL <---  (%d. aiss16ao2_ioctl() failure, error %d)\n",
				__LINE__,
				ret);
	}
	else if ((value < VREF_MIN) || (value > VREF_MAX))
	{
		errs++;
		printf(	"FAIL <---  (%d. Vref error: 0x%04lX)\n",
				__LINE__,
				(long) value);
	}

	errs	+= aiss16ao2_initialize(fd, -1, 0);
	return(errs);
}



/******************************************************************************
*
*	Function:	ai_mode_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service AISS16AO2_IOCTL_AI_MODE.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int ai_mode_test(int fd)
{
	int	errs	= 0;

	gsc_label("AISS16AO2_IOCTL_AI_MODE");
	errs	+= _service_test(fd);
	errs	+= _function_test(fd);

	if (errs == 0)
		printf("PASS\n");

	return(errs);
}


