// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AISS16AO2/samples/sbtest/ai_range_b.c $
// $Rev: 52930 $
// $Date: 2023-04-20 15:46:28 -0500 (Thu, 20 Apr 2023) $

// 16AISS16AO2: Sample Application: source file

#include "main.h"



// macros *********************************************************************

// The output voltage is 2.0 volts.
#define	VOUT_2_RANGE_2_5	0xE666
#define	VIN_2_RANGE_2_5		0xE666
#define	VIN_2_RANGE_5		0xB332
#define	VIN_2_RANGE_10		0x9999

#define	MAX_2_5V			(VIN_2_RANGE_2_5	+ 1311)	// +-0.1 volts
#define	MIN_2_5V			(VIN_2_RANGE_2_5	- 1311)
#define	MAX_5V				(VIN_2_RANGE_5		+ 655)	// +-0.1 volts
#define	MIN_5V				(VIN_2_RANGE_5		- 655)
#define	MAX_10V				(VIN_2_RANGE_10		+ 328)	// +-0.1 volts
#define	MIN_10V				(VIN_2_RANGE_10		- 328)


//*****************************************************************************
static int _service_test(int fd)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_AI_RANGE_B,
			/* arg		*/	AISS16AO2_RANGE_2_5V,
			/* reg		*/	AISS16AO2_GSC_BCR,
			/* mask		*/	0xC0,
			/* value	*/	0x00
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_AI_RANGE_B,
			/* arg		*/	AISS16AO2_RANGE_5V,
			/* reg		*/	AISS16AO2_GSC_BCR,
			/* mask		*/	0xC0,
			/* value	*/	0x40
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_AI_RANGE_B,
			/* arg		*/	AISS16AO2_RANGE_10V,
			/* reg		*/	AISS16AO2_GSC_BCR,
			/* mask		*/	0xC0,
			/* value	*/	0x80
		},

		{ SERVICE_END_LIST }
	};

	int errs	= 0;

	errs	+= aiss16ao2_initialize(fd, -1, 0);

	errs	+= service_ioctl_set_reg_list(fd, list);
	errs	+= service_ioctl_reg_get_list(fd, list);

	errs	+= service_ioctl_set_reg_list(fd, list);
	errs	+= service_ioctl_reg_get_list(fd, list);

	errs	+= aiss16ao2_initialize(fd, -1, 0);
	return(errs);
}




//*****************************************************************************
static int _function_test(int fd)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_AI_CLOCK_MODE,
			/* arg		*/	AISS16AO2_CLOCK_MODE_OUT,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_TRIGGER_MODE,
			/* arg		*/	AISS16AO2_TRIGGER_MODE_OUT,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		// Configure the outputs.

		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_AO_MODE,
			/* arg		*/	AISS16AO2_AO_MODE_REG,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_AO_OUTPUT_MODE,
			/* arg		*/	AISS16AO2_AO_OUTPUT_MODE_IMMED,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_AO_RANGE,
			/* arg		*/	AISS16AO2_RANGE_2_5V,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		// Configure the inputs.

		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_GEN_A_NDIV,
			/* arg		*/	90,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_GEN_A_ENABLE,
			/* arg		*/	AISS16AO2_GEN_ENABLE_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_AI_FSAMP_RANGE,
			/* arg		*/	AISS16AO2_FSAMP_RANGE_LOW,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_AI_CHAN_SEL,
			/* arg		*/	0x4,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	100,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_AI_ENABLE,
			/* arg		*/	AISS16AO2_AIO_ENABLE_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	static const service_data_t	_2_5v[]	=
	{
		// Verify the 2.5 analog input range.

		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_AI_MODE,
			/* arg		*/	AISS16AO2_AI_MODE_AO_0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_AI_RANGE_B,
			/* arg		*/	AISS16AO2_RANGE_2_5V,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_AUTO_CALIBRATE,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_REG_MOD,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	AISS16AO2_GSC_AOC0R,
			/* mask		*/	0xFFFF,
			/* value	*/	VOUT_2_RANGE_2_5
		},
		{
			/* service	*/	SERVICE_REG_MOD,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	AISS16AO2_GSC_AOC0R,
			/* mask		*/	0xFFFF,
			/* value	*/	VOUT_2_RANGE_2_5
		},
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	100,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_AI_BUF_CLEAR,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	10,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	static const service_data_t	_5v[]	=
	{
		// Verify the 5 analog input range.

		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_AI_MODE,
			/* arg		*/	AISS16AO2_AI_MODE_AO_0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_AI_RANGE_B,
			/* arg		*/	AISS16AO2_RANGE_5V,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_AUTO_CALIBRATE,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_REG_MOD,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	AISS16AO2_GSC_AOC0R,
			/* mask		*/	0xFFFF,
			/* value	*/	VOUT_2_RANGE_2_5
		},
		{
			/* service	*/	SERVICE_REG_MOD,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	AISS16AO2_GSC_AOC0R,
			/* mask		*/	0xFFFF,
			/* value	*/	VOUT_2_RANGE_2_5
		},
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	100,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_AI_BUF_CLEAR,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	10,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	static const service_data_t	_10v[]	=
	{
		// Verify the 10 analog input range.

		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_AI_MODE,
			/* arg		*/	AISS16AO2_AI_MODE_AO_0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_AI_RANGE_B,
			/* arg		*/	AISS16AO2_RANGE_10V,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_AUTO_CALIBRATE,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_REG_MOD,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	AISS16AO2_GSC_AOC0R,
			/* mask		*/	0xFFFF,
			/* value	*/	VOUT_2_RANGE_2_5
		},
		{
			/* service	*/	SERVICE_REG_MOD,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	AISS16AO2_GSC_AOC0R,
			/* mask		*/	0xFFFF,
			/* value	*/	VOUT_2_RANGE_2_5
		},
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	100,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_AI_BUF_CLEAR,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	10,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	int			errs	= 0;
	gsc_reg_t	parm;
	int			ret;
	s32			value;

	errs	+= aiss16ao2_initialize(fd, -1, 0);

	errs	+= service_ioctl_set_reg_list(fd, list);

	// Verify the 2.5 V input range.
	errs		+= service_ioctl_set_reg_list(fd, _2_5v);
	parm.reg	= AISS16AO2_GSC_AIBR;
	parm.value	= 0xDEADBEEF;
	parm.mask	= 0xBEEFDEAD;
	ret			= aiss16ao2_ioctl(fd, AISS16AO2_IOCTL_REG_READ, (void*) &parm);
	value		= parm.value & 0xFFFF;

	if (ret)
	{
		errs++;
		printf(	"FAIL <---  (%d. aiss16ao2_ioctl() failure, error %d)\n",
				__LINE__,
				ret);
	}
	else if ((value < MIN_2_5V) || (value > MAX_2_5V))
	{
		errs++;
		printf(	"FAIL <---  (%d. 2.5 V range error: 0x%04lX)\n",
				__LINE__,
				(long) value);
	}

	// Verify the 5 V input range.
	errs		+= service_ioctl_set_reg_list(fd, _5v);
	parm.reg	= AISS16AO2_GSC_AIBR;
	parm.value	= 0xDEADBEEF;
	parm.mask	= 0xBEEFDEAD;
	ret			= aiss16ao2_ioctl(fd, AISS16AO2_IOCTL_REG_READ, (void*) &parm);
	value		= parm.value & 0xFFFF;

	if (ret < 0)
	{
		errs++;
		printf(	"FAIL <---  (%d. aiss16ao2_ioctl() failure, error %d)\n",
				__LINE__,
				ret);
	}
	else if ((value < MIN_5V) || (value > MAX_5V))
	{
		errs++;
		printf(	"FAIL <---  (%d. 5 V range error: 0x%04lX)\n",
				__LINE__,
				(long) value);
	}

	// Verify the 10 V input range.
	errs		+= service_ioctl_set_reg_list(fd, _10v);
	parm.reg	= AISS16AO2_GSC_AIBR;
	parm.value	= 0xDEADBEEF;
	parm.mask	= 0xBEEFDEAD;
	ret			= aiss16ao2_ioctl(fd, AISS16AO2_IOCTL_REG_READ, (void*) &parm);
	value		= parm.value & 0xFFFF;

	if (ret < 0)
	{
		errs++;
		printf(	"FAIL <---  (%d. aiss16ao2_ioctl() failure, error %d)\n",
				__LINE__,
				ret);
	}
	else if ((value < MIN_10V) || (value > MAX_10V))
	{
		errs++;
		printf(	"FAIL <---  (%d. 10 V range error: 0x%04lX)\n",
				__LINE__,
				(long) value);
	}

	errs	+= aiss16ao2_initialize(fd, -1, 0);
	return(errs);
}




/******************************************************************************
*
*	Function:	ai_range_b_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service AISS16AO2_IOCTL_AI_RANGE_B.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int ai_range_b_test(int fd)
{
	int	errs	= 0;

	gsc_label("AISS16AO2_IOCTL_AI_RANGE_B");
	errs	+= _service_test(fd);
	errs	+= _function_test(fd);

	if (errs == 0)
		printf("PASS\n");

	return(errs);
}


