// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AISS16AO2/samples/sbtest/ao_buf_clear.c $
// $Rev: 52930 $
// $Date: 2023-04-20 15:46:28 -0500 (Thu, 20 Apr 2023) $

// 16AISS16AO2: Sample Application: source file

#include "main.h"



// variables ******************************************************************

static	u32	_buffer[64L * 1024];



//*****************************************************************************
static int _service_test(int fd)
{
	int errs	= 0;

	// There are no persistent bits to test with this service.
	return(errs);
}



//*****************************************************************************
static int _function_test(int fd)
{
	static const service_data_t	list_1[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_AO_MODE,
			/* arg		*/	AISS16AO2_AO_MODE_FIFO,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_TX_IO_MODE,
			/* arg		*/	GSC_IO_MODE_PIO,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_TX_IO_OVERFLOW,
			/* arg		*/	AISS16AO2_IO_OVERFLOW_IGNORE,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	static const service_data_t	list_2[]	=
	{
		// Verify that the buffer is full.

		{
			/* service	*/	SERVICE_REG_TEST,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	AISS16AO2_GSC_OBSR,
			/* mask		*/	0x7FFFF,
			/* value	*/	0x40000
		},

		// Force a buffer overflow.

		{					// Write to the output buffer.
			/* service	*/	SERVICE_REG_MOD,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	AISS16AO2_GSC_AOBR,
			/* mask		*/	0,
			/* value	*/	0
		},
		{					// Verify that we got an overflow.
			/* service	*/	SERVICE_REG_TEST,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	AISS16AO2_GSC_BOOR,
			/* mask		*/	0x10000,
			/* value	*/	0x10000
		},

		// Force a frame overflow.

		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_AO_BUF_MODE,
			/* arg		*/	AISS16AO2_BUF_MODE_CIRC,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{					// Write to the output buffer.
			/* service	*/	SERVICE_REG_MOD,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	AISS16AO2_GSC_AOBR,
			/* mask		*/	0,
			/* value	*/	0
		},
		{					// Verify that we got an overflow.
			/* service	*/	SERVICE_REG_TEST,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	AISS16AO2_GSC_BOOR,
			/* mask		*/	0x20000,
			/* value	*/	0x20000
		},

		// Clear the buffer and verify the results.

		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_AO_BUF_CLEAR,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{					// Verify that the buffer is empty.
			/* service	*/	SERVICE_REG_TEST,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	AISS16AO2_GSC_OBSR,
			/* mask		*/	0x7FFFF,
			/* value	*/	0x00000
		},
		{					// Verify that there is no data overflow.
			/* service	*/	SERVICE_REG_TEST,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	AISS16AO2_GSC_BOOR,
			/* mask		*/	0x10000,
			/* value	*/	0x00000
		},
		{					// Verify that there is no frame overflow.
			/* service	*/	SERVICE_REG_TEST,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	AISS16AO2_GSC_BOOR,
			/* mask		*/	0x20000,
			/* value	*/	0x00000
		},

		{ SERVICE_END_LIST }
	};

	int		errs	= 0;
	int		ret;

	errs	+= aiss16ao2_initialize(fd, -1, 0);
	errs	+= service_ioctl_set_reg_list(fd, list_1);

	memset(_buffer, 0xFF, sizeof(_buffer));

	ret		= aiss16ao2_write(fd, _buffer, sizeof(_buffer));
	errs	+= (ret < 0) ? 1 : 0;

	ret		= aiss16ao2_write(fd, _buffer, sizeof(_buffer));
	errs	+= (ret < 0) ? 1 : 0;

	ret		= aiss16ao2_write(fd, _buffer, sizeof(_buffer));
	errs	+= (ret < 0) ? 1 : 0;

	ret		= aiss16ao2_write(fd, _buffer, sizeof(_buffer));
	errs	+= (ret < 0) ? 1 : 0;

	errs	+= service_ioctl_set_reg_list(fd, list_2);

	errs	+= aiss16ao2_initialize(fd, -1, 0);
	return(errs);
}



/******************************************************************************
*
*	Function:	ao_buf_clear_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service AISS16AO2_IOCTL_AO_BUF_CLEAR.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int ao_buf_clear_test(int fd)
{
	int	errs	= 0;

	gsc_label("AISS16AO2_IOCTL_AO_BUF_CLEAR");
	errs	+= _service_test(fd);
	errs	+= _function_test(fd);

	if (errs == 0)
		printf("PASS\n");

	return(errs);
}


