// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AISS16AO2/samples/sbtest/ao_buf_over_fram.c $
// $Rev: 52930 $
// $Date: 2023-04-20 15:46:28 -0500 (Thu, 20 Apr 2023) $

// 16AISS16AO2: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _service_test(int fd)
{
	// There are no persistent bits to test with this service.
	return(0);
}




//*****************************************************************************
static int _function_test(int fd)
{
	static const service_data_t	list_1[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_AO_MODE,
			/* arg		*/	AISS16AO2_AO_MODE_FIFO,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_AO_BUF_MODE,
			/* arg		*/	AISS16AO2_BUF_MODE_CIRC,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	static const service_data_t	list_2[]	=
	{
		{					// Force a buffer overflow.
			/* service	*/	SERVICE_REG_MOD,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	AISS16AO2_GSC_AOBR,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	int	errs	= 0;
	int	ret;
	s32	val;

	errs	+= aiss16ao2_initialize(fd, -1, 0);
	errs	+= service_ioctl_set_reg_list(fd, list_1);

	val		= -1;
	ret		= aiss16ao2_ioctl(fd, AISS16AO2_IOCTL_AO_BUF_OVER_FRAM, &val);
	errs	+= ret ? 1 : 0;

	if ((errs == 0) && (val != AISS16AO2_BUF_OVERFLOW_NO))
	{
		errs++;
		printf(	"FAIL <---  (%d. There is an unexpected overflow.)\n",
				__LINE__);
	}

	errs	+= service_ioctl_set_reg_list(fd, list_2);

	val		= -1;
	ret		= aiss16ao2_ioctl(fd, AISS16AO2_IOCTL_AO_BUF_OVER_FRAM, &val);
	errs	+= ret ? 1 : 0;

	if ((errs == 0) && (val != AISS16AO2_BUF_OVERFLOW_YES))
	{
		errs++;
		printf(	"FAIL <---  (%d. Overflow expected but did not occur.)\n",
				__LINE__);
	}

	val		= AISS16AO2_BUF_OVERFLOW_TEST;
	ret		= aiss16ao2_ioctl(fd, AISS16AO2_IOCTL_AO_BUF_OVER_FRAM, &val);
	errs	+= ret ? 1 : 0;

	if ((errs == 0) && (val != AISS16AO2_BUF_OVERFLOW_YES))
	{
		errs++;
		printf(	"FAIL <---  (%d. An overflow was unexpectedly cleared.)\n",
				__LINE__);
	}

	val		= AISS16AO2_BUF_OVERFLOW_CLEAR;
	ret		= aiss16ao2_ioctl(fd, AISS16AO2_IOCTL_AO_BUF_OVER_FRAM, &val);
	errs	+= ret ? 1 : 0;

	if ((errs == 0) && (val != AISS16AO2_BUF_OVERFLOW_NO))
	{
		errs++;
		printf(	"FAIL <---  (%d. An overflow was not cleared.)\n",
				__LINE__);
	}

	errs	+= aiss16ao2_initialize(fd, -1, 0);
	return(errs);
}




/******************************************************************************
*
*	Function:	ao_buf_over_fram_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service AISS16AO2_IOCTL_AO_BUF_OVER_FRAM.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int ao_buf_over_fram_test(int fd)
{
	int	errs		= 0;

	gsc_label("AISS16AO2_IOCTL_AO_BUF_OVER_FRAM");
	errs	+= _service_test(fd);
	errs	+= _function_test(fd);

	if (errs == 0)
		printf("PASS\n");

	return(errs);
}


