// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AISS16AO2/samples/sbtest/ao_output_mode.c $
// $Rev: 52930 $
// $Date: 2023-04-20 15:46:28 -0500 (Thu, 20 Apr 2023) $

// 16AISS16AO2: Sample Application: source file

#include "main.h"



// macros *********************************************************************

#define	IMMED		0xA666			// 3.0 volts
#define	SIMUL		0xCCCC			// 6.0 volts
#define	IMMED_MAX	(IMMED + 328)	// +-0.1 volts
#define	IMMED_MIN	(IMMED - 328)
#define	SIMUL_MAX	(SIMUL + 328)	// +-0.1 volts
#define	SIMUL_MIN	(SIMUL - 328)



//*****************************************************************************
static int _service_test(int fd)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_AO_OUTPUT_MODE,
			/* arg		*/	AISS16AO2_AO_OUTPUT_MODE_SIMUL,
			/* reg		*/	AISS16AO2_GSC_BCR,
			/* mask		*/	0x40000,
			/* value	*/	0x40000
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_AO_OUTPUT_MODE,
			/* arg		*/	AISS16AO2_AO_OUTPUT_MODE_IMMED,
			/* reg		*/	AISS16AO2_GSC_BCR,
			/* mask		*/	0x40000,
			/* value	*/	0x00000
		},

		{ SERVICE_END_LIST }
	};

	int errs	= 0;

	errs	+= aiss16ao2_initialize(fd, -1, 0);

	errs	+= service_ioctl_set_reg_list(fd, list);
	errs	+= service_ioctl_reg_get_list(fd, list);

	errs	+= service_ioctl_set_reg_list(fd, list);
	errs	+= service_ioctl_reg_get_list(fd, list);

	errs	+= aiss16ao2_initialize(fd, -1, 0);
	return(errs);
}




//*****************************************************************************
static int _function_test(int fd)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_DATA_FORMAT,
			/* arg		*/	AISS16AO2_DATA_FORMAT_OFF_BIN,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_AI_CLOCK_MODE,
			/* arg		*/	AISS16AO2_CLOCK_MODE_OUT,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_AO_CLOCK_MODE,
			/* arg		*/	AISS16AO2_CLOCK_MODE_OUT,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		// Configure the outputs.

		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_AO_RANGE,
			/* arg		*/	AISS16AO2_RANGE_10V,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_AO_MODE,
			/* arg		*/	AISS16AO2_AO_MODE_REG,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_GEN_C_ENABLE,
			/* arg		*/	AISS16AO2_GEN_ENABLE_NO,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_AO_ENABLE,
			/* arg		*/	AISS16AO2_AIO_ENABLE_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		// Configure the inputs.

		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_AI_RANGE_A,
			/* arg		*/	AISS16AO2_RANGE_10V,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_GEN_A_NDIV,
			/* arg		*/	90,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_GEN_A_ENABLE,
			/* arg		*/	AISS16AO2_GEN_ENABLE_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_AI_FSAMP_RANGE,
			/* arg		*/	AISS16AO2_FSAMP_RANGE_LOW,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_AI_CHAN_SEL,
			/* arg		*/	0x1,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	100,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_AI_ENABLE,
			/* arg		*/	AISS16AO2_AIO_ENABLE_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	10,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};


	static const service_data_t	immed[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_AO_OUTPUT_MODE,
			/* arg		*/	AISS16AO2_AO_OUTPUT_MODE_IMMED,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_AI_MODE,
			/* arg		*/	AISS16AO2_AI_MODE_AO_0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_AUTO_CALIBRATE,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_AO_BUF_CLEAR,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_REG_MOD,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	AISS16AO2_GSC_AOC0R,
			/* mask		*/	0xFFFF,
			/* value	*/	IMMED
		},
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	100,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_AI_BUF_CLEAR,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	10,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	static const service_data_t	simul_1[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_AO_OUTPUT_MODE,
			/* arg		*/	AISS16AO2_AO_OUTPUT_MODE_SIMUL,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	10,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_REG_MOD,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	AISS16AO2_GSC_AOC0R,
			/* mask		*/	0xFFFF,
			/* value	*/	SIMUL
		},
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	100,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_AI_BUF_CLEAR,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	10,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	static const service_data_t	simul_2[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_AO_SW_CLOCK,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	100,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_AI_BUF_CLEAR,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	10,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	int			errs	= 0;
	gsc_reg_t	parm;
	int			ret;
	s32			value;

	errs	+= aiss16ao2_initialize(fd, -1, 0);

	errs	+= service_ioctl_set_reg_list(fd, list);

	// Verify the IMMEDIATE option.
	errs		+= service_ioctl_set_reg_list(fd, immed);
	parm.reg	= AISS16AO2_GSC_AIBR;
	parm.value	= 0xDEADBEEF;
	parm.mask	= 0xBEEFDEAD;
	ret			= aiss16ao2_ioctl(fd, AISS16AO2_IOCTL_REG_READ, (void*) &parm);
	value		= parm.value & 0xFFFF;

	if (ret < 0)
	{
		errs++;
		printf(	"FAIL <---  (%d. aiss16ao2_ioctl() failure, error %d)\n",
				__LINE__,
				ret);
	}
	else if ((value < IMMED_MIN) || (value > IMMED_MAX))
	{
		errs++;
		printf(	"FAIL <---  (%d. IMMEDIATE option error: 0x%04lX, %.3f)\n",
				__LINE__,
				(long) value,
				((float) value / 65535) * 20 - 10);
	}

	// Verify the SIMULTANEOUS option - without clock.
	errs		+= service_ioctl_set_reg_list(fd, simul_1);
	parm.reg	= AISS16AO2_GSC_AIBR;
	parm.value	= 0xDEADBEEF;
	parm.mask	= 0xBEEFDEAD;
	ret			= aiss16ao2_ioctl(fd, AISS16AO2_IOCTL_REG_READ, (void*) &parm);
	value		= parm.value & 0xFFFF;

	if (ret < 0)
	{
		errs++;
		printf(	"FAIL <---  (%d. aiss16ao2_ioctl() failure, error %d)\n",
				__LINE__,
				ret);
	}
	else if ((value < IMMED_MIN) || (value > IMMED_MAX))
	{
		errs++;
		printf(	"FAIL <---  (%d. SIMULTANEOUS option error: 0x%04lX, %.3f)\n",
				__LINE__,
				(long) value,
				((float) value / 65535) * 20 - 10);
	}

	// Verify the SIMULTANEOUS option - with clock.
	errs		+= service_ioctl_set_reg_list(fd, simul_2);
	parm.reg	= AISS16AO2_GSC_AIBR;
	parm.value	= 0xDEADBEEF;
	parm.mask	= 0xBEEFDEAD;
	ret			= aiss16ao2_ioctl(fd, AISS16AO2_IOCTL_REG_READ, (void*) &parm);
	value		= parm.value & 0xFFFF;

	if (ret < 0)
	{
		errs++;
		printf(	"FAIL <---  (%d. aiss16ao2_ioctl() failure, error %d)\n",
				__LINE__,
				ret);
	}
	else if ((value < SIMUL_MIN) || (value > SIMUL_MAX))
	{
		errs++;
		printf(	"FAIL <---  (%d. SIMULTANEOUS option error: 0x%04lX, %.3f)\n",
				__LINE__,
				(long) value,
				((float) value / 65535) * 20 - 10);
	}

	errs	+= aiss16ao2_initialize(fd, -1, 0);
	return(errs);
}




/******************************************************************************
*
*	Function:	ao_output_mode_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service AISS16AO2_IOCTL_AO_OUTPUT_MODE.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int ao_output_mode_test(int fd)
{
	int	errs	= 0;

	gsc_label("AISS16AO2_IOCTL_AO_OUTPUT_MODE");
	errs	+= _service_test(fd);
	errs	+= _function_test(fd);

	if (errs == 0)
		printf("PASS\n");

	return(errs);
}


