// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AISS16AO2/samples/sbtest/ao_range.c $
// $Rev: 52930 $
// $Date: 2023-04-20 15:46:28 -0500 (Thu, 20 Apr 2023) $

// 16AISS16AO2: Sample Application: source file

#include "main.h"



// macros *********************************************************************

#define	RANGE_2_5_V			0xE666		// 2.0 volts
#define	RANGE_5_V			0xE666		// 4.0 volts
#define	RANGE_10_V			0xE666		// 8.0 volts

#define	RANGE_10_MAX_2_5V	((long) (((2.0 + 0.1 + 10) / 20) * 65535))	// +-0.1 volts
#define	RANGE_10_MIN_2_5V	((long) (((2.0 - 0.1 + 10) / 20) * 65535))
#define	RANGE_10_MAX_5V		((long) (((4.0 + 0.1 + 10) / 20) * 65535))	// +-0.1 volts
#define	RANGE_10_MIN_5V		((long) (((4.0 - 0.1 + 10) / 20) * 65535))
#define	RANGE_10_MAX_10V	((long) (((8.0 + 0.1 + 10) / 20) * 65535))	// +-0.1 volts
#define	RANGE_10_MIN_10V	((long) (((8.0 - 0.1 + 10) / 20) * 65535))



//*****************************************************************************
static int _service_test(int fd)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_AO_RANGE,
			/* arg		*/	AISS16AO2_RANGE_2_5V,
			/* reg		*/	AISS16AO2_GSC_BCR,
			/* mask		*/	0x30000,
			/* value	*/	0x00000
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_AO_RANGE,
			/* arg		*/	AISS16AO2_RANGE_5V,
			/* reg		*/	AISS16AO2_GSC_BCR,
			/* mask		*/	0x30000,
			/* value	*/	0x10000
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_AO_RANGE,
			/* arg		*/	AISS16AO2_RANGE_10V,
			/* reg		*/	AISS16AO2_GSC_BCR,
			/* mask		*/	0x30000,
			/* value	*/	0x20000
		},

		{ SERVICE_END_LIST }
	};

	int errs	= 0;

	errs	+= aiss16ao2_initialize(fd, -1, 0);

	errs	+= service_ioctl_set_reg_list(fd, list);
	errs	+= service_ioctl_reg_get_list(fd, list);

	errs	+= service_ioctl_set_reg_list(fd, list);
	errs	+= service_ioctl_reg_get_list(fd, list);

	errs	+= aiss16ao2_initialize(fd, -1, 0);
	return(errs);
}




//*****************************************************************************
static int _function_test(int fd)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_AI_CLOCK_MODE,
			/* arg		*/	AISS16AO2_CLOCK_MODE_OUT,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_TRIGGER_MODE,
			/* arg		*/	AISS16AO2_TRIGGER_MODE_OUT,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		// Configure the outputs.

		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_AO_MODE,
			/* arg		*/	AISS16AO2_AO_MODE_REG,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_AO_OUTPUT_MODE,
			/* arg		*/	AISS16AO2_AO_OUTPUT_MODE_IMMED,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_AI_RANGE_A,
			/* arg		*/	AISS16AO2_RANGE_10V,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		// Configure the inputs.

		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_GEN_A_NDIV,
			/* arg		*/	90,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_GEN_A_ENABLE,
			/* arg		*/	AISS16AO2_GEN_ENABLE_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_AI_FSAMP_RANGE,
			/* arg		*/	AISS16AO2_FSAMP_RANGE_LOW,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_AI_CHAN_SEL,
			/* arg		*/	0x1,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	100,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_AI_ENABLE,
			/* arg		*/	AISS16AO2_AIO_ENABLE_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	10,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	static const service_data_t	_2_5v[]	=
	{
		// Verify the 2.5 analog input range.

		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_AI_MODE,
			/* arg		*/	AISS16AO2_AI_MODE_AO_0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_AO_RANGE,
			/* arg		*/	AISS16AO2_RANGE_2_5V,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_AUTO_CALIBRATE,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_REG_MOD,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	AISS16AO2_GSC_AOC0R,
			/* mask		*/	0xFFFF,
			/* value	*/	RANGE_2_5_V
		},
		{
			/* service	*/	SERVICE_REG_MOD,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	AISS16AO2_GSC_AOC0R,
			/* mask		*/	0xFFFF,
			/* value	*/	RANGE_2_5_V
		},
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	100,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_AI_BUF_CLEAR,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	10,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	static const service_data_t	_5v[]	=
	{
		// Verify the 5 analog input range.

		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_AI_MODE,
			/* arg		*/	AISS16AO2_AI_MODE_AO_0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_AO_RANGE,
			/* arg		*/	AISS16AO2_RANGE_5V,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_AUTO_CALIBRATE,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_REG_MOD,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	AISS16AO2_GSC_AOC0R,
			/* mask		*/	0xFFFF,
			/* value	*/	RANGE_5_V
		},
		{
			/* service	*/	SERVICE_REG_MOD,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	AISS16AO2_GSC_AOC0R,
			/* mask		*/	0xFFFF,
			/* value	*/	RANGE_5_V
		},
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	100,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_AI_BUF_CLEAR,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	10,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	static const service_data_t	_10v[]	=
	{
		// Verify the 10 analog input range.

		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_AI_MODE,
			/* arg		*/	AISS16AO2_AI_MODE_AO_0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_AO_RANGE,
			/* arg		*/	AISS16AO2_RANGE_10V,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_AUTO_CALIBRATE,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_REG_MOD,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	AISS16AO2_GSC_AOC0R,
			/* mask		*/	0xFFFF,
			/* value	*/	RANGE_10_V
		},
		{
			/* service	*/	SERVICE_REG_MOD,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	AISS16AO2_GSC_AOC0R,
			/* mask		*/	0xFFFF,
			/* value	*/	RANGE_10_V
		},
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	100,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AISS16AO2_IOCTL_AI_BUF_CLEAR,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	10,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	int			errs	= 0;
	gsc_reg_t	parm;
	int			ret;
	s32			value;

	errs	+= aiss16ao2_initialize(fd, -1, 0);

	errs	+= service_ioctl_set_reg_list(fd, list);

	// Verify the 2.5 V output range.
	errs		+= service_ioctl_set_reg_list(fd, _2_5v);
	parm.reg	= AISS16AO2_GSC_AIBR;
	parm.value	= 0xDEADBEEF;
	parm.mask	= 0xBEEFDEAD;
	ret			= aiss16ao2_ioctl(fd, AISS16AO2_IOCTL_REG_READ, (void*) &parm);
	value		= parm.value & 0xFFFF;

	if (ret < 0)
	{
		errs++;
		printf(	"FAIL <---  (%d. aiss16ao2_ioctl() failure, error %d)\n",
				__LINE__,
				ret);
	}
	else if ((value < RANGE_10_MIN_2_5V) || (value > RANGE_10_MAX_2_5V))
	{
		errs++;
		printf(	"FAIL <---  (%d. 2.5 V range error:"
				" got 0x%lX, min 0x%lX, max 0x%04lX)\n",
				__LINE__,
				(long) value,
				(long) RANGE_10_MIN_2_5V,
				(long) RANGE_10_MAX_2_5V);
	}

	// Verify the 5 V output range.
	errs		+= service_ioctl_set_reg_list(fd, _5v);
	parm.reg	= AISS16AO2_GSC_AIBR;
	parm.value	= 0xDEADBEEF;
	parm.mask	= 0xBEEFDEAD;
	ret			= aiss16ao2_ioctl(fd, AISS16AO2_IOCTL_REG_READ, (void*) &parm);
	value		= parm.value & 0xFFFF;

	if (ret < 0)
	{
		errs++;
		printf(	"FAIL <---  (%d. aiss16ao2_ioctl() failure, error %d)\n",
				__LINE__,
				ret);
	}
	else if ((value < RANGE_10_MIN_5V) || (value > RANGE_10_MAX_5V))
	{
		errs++;
		printf(	"FAIL <---  (%d. 2 V range error:"
				" got 0x%lX, min 0x%lX, max 0x%04lX)\n",
				__LINE__,
				(long) value,
				(long) RANGE_10_MIN_5V,
				(long) RANGE_10_MAX_5V);
	}

	// Verify the 10 V output range.
	errs		+= service_ioctl_set_reg_list(fd, _10v);
	parm.reg	= AISS16AO2_GSC_AIBR;
	parm.value	= 0xDEADBEEF;
	parm.mask	= 0xBEEFDEAD;
	ret			= aiss16ao2_ioctl(fd, AISS16AO2_IOCTL_REG_READ, (void*) &parm);
	value		= parm.value & 0xFFFF;

	if (ret < 0)
	{
		errs++;
		printf(	"FAIL <---  (%d. aiss16ao2_ioctl() failure, error %d)\n",
				__LINE__,
				ret);
	}
	else if ((value < RANGE_10_MIN_10V) || (value > RANGE_10_MAX_10V))
	{
		errs++;
		printf(	"FAIL <---  (%d. 10 V range error:"
				" got 0x%lX, min 0x%lX, max 0x%04lX)\n",
				__LINE__,
				(long) value,
				(long) RANGE_10_MIN_10V,
				(long) RANGE_10_MAX_10V);
	}

	errs	+= aiss16ao2_initialize(fd, -1, 0);
	return(errs);
}




/******************************************************************************
*
*	Function:	ao_range_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service AISS16AO2_IOCTL_AO_RANGE.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int ao_range_test(int fd)
{
	int	errs	= 0;

	gsc_label("AISS16AO2_IOCTL_AO_RANGE");
	errs	+= _service_test(fd);
	errs	+= _function_test(fd);

	if (errs == 0)
		printf("PASS\n");

	return(errs);
}



