// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AISS16AO2/samples/sbtest/reg_mod.c $
// $Rev: 52930 $
// $Date: 2023-04-20 15:46:28 -0500 (Thu, 20 Apr 2023) $

// 16AISS16AO2: Sample Application: source file

#include "main.h"



/******************************************************************************
*
*	Function:	reg_mod_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service AISS16AO2_IOCTL_REG_MOD.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int reg_mod_test(int fd)
{
	static const struct
	{
		u32	reg;
		u32	mask;
	} list[]	=
	{
		// reg					mask
		{ AISS16AO2_GSC_IBTR,	0x0003FFFF	},
		{ AISS16AO2_GSC_OBTR,	0x0003FFFF	},
	};

	u32			bit;
	int			errs	= 0;
	int			i;
	gsc_reg_t	parm;
	int			ret;
	u32			value;

	gsc_label("AISS16AO2_IOCTL_REG_MOD");

	for (i = 0; (errs == 0) && (i < SIZEOF_ARRAY(list)); i++)
	{

		for (bit = 0x1; bit; bit <<= 1)
		{
			if ((list[i].mask & bit) == 0)
				continue;

			// Read the register value so we can restore it later.
			parm.reg	= list[i].reg;
			ret			= aiss16ao2_ioctl(fd, AISS16AO2_IOCTL_REG_READ, (void*) &parm);
			value		= parm.value;

			if (ret < 0)
			{
				errs	= 1;
				printf(	"FAIL <---  (%d. i %d, aiss16ao2_ioctl() failure, error %d)\n",
						__LINE__,
						i,
						ret);
				break;
			}

			// Set the bit low.
			parm.value	= 0;
			parm.mask	= bit;
			ret			= aiss16ao2_ioctl(fd, AISS16AO2_IOCTL_REG_MOD, (void*) &parm);

			if (ret < 0)
			{
				errs	= 1;
				printf(	"FAIL <---  (%d. i %d, aiss16ao2_ioctl() failure, error %d)\n",
						__LINE__,
						i,
						ret);
				break;
			}

			// Now verify that the bit is low.
			ret	= aiss16ao2_ioctl(fd, AISS16AO2_IOCTL_REG_READ, (void*) &parm);

			if (ret < 0)
			{
				errs	= 1;
				printf(	"FAIL <---  (%d. i %d, aiss16ao2_ioctl() failure, error %d)\n",
						__LINE__,
						i,
						ret);
				break;
			}

			if (parm.value & bit)
			{
				errs	= 1;
				printf(	"FAIL <---  (%d. i %d, bit not low, mask 0x%lX, got 0x%lX)\n",
						__LINE__,
						i,
						(long) bit,
						(long) parm.value);
				break;
			}

			// Now restore the register to its previous value.
			parm.value	= value;
			ret			= aiss16ao2_ioctl(fd, AISS16AO2_IOCTL_REG_WRITE, (void*) &parm);

			if (ret < 0)
			{
				errs	= 1;
				printf(	"FAIL <---  (%d. i %d, aiss16ao2_ioctl() failure, error %d)\n",
						__LINE__,
						i,
						ret);
				break;
			}

			// Set the bit high.
			parm.value	= bit;
			parm.mask	= bit;
			ret			= aiss16ao2_ioctl(fd, AISS16AO2_IOCTL_REG_MOD, (void*) &parm);

			if (ret < 0)
			{
				errs	= 1;
				printf(	"FAIL <---  (%d. i %d, aiss16ao2_ioctl() failure, error %d)\n",
						__LINE__,
						i,
						ret);
				break;
			}

			// Now verify that the bit is high.
			ret	= aiss16ao2_ioctl(fd, AISS16AO2_IOCTL_REG_READ, (void*) &parm);

			if (ret < 0)
			{
				errs	= 1;
				printf(	"FAIL <---  (%d. i %d, aiss16ao2_ioctl() failure, error %d)\n",
						__LINE__,
						i,
						ret);
				break;
			}

			if ((parm.value & bit) == 0)
			{
				errs	= 1;
				printf(	"FAIL <---  (%d. i %d, bit not high, mask 0x%lX, got 0x%lX)\n",
						__LINE__,
						i,
						(long) bit,
						(long) parm.value);
				break;
			}

			// Now restore the register to its previous value.
			parm.value	= value;
			ret			= aiss16ao2_ioctl(fd, AISS16AO2_IOCTL_REG_WRITE, (void*) &parm);

			if (ret < 0)
			{
				errs	= 1;
				printf(	"FAIL <---  (%d. i %d, aiss16ao2_ioctl() failure, error %d)\n",
						__LINE__,
						i,
						ret);
				break;
			}
		}
	}

	if (errs == 0)
		printf("PASS\n");

	return(errs);
}


