// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AISS16AO2/utils/util_ai_fsamp_range.c $
// $Rev: 44439 $
// $Date: 2019-02-13 10:58:37 -0600 (Wed, 13 Feb 2019) $

// 16AISS16AO2: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	aiss16ao2_ai_fsamp_range
*
*	Purpose:
*
*		Provide a visual wrapper for the AISS16AO2_IOCTL_AI_FSAMP_RANGE service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		set		This is the value to apply.
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int	aiss16ao2_ai_fsamp_range(int fd, int index, int verbose, s32 set, s32* get)
{
	char		buf[128];
	int			errs;
	const char*	ptr		= buf;
	int			ret;

	if (verbose)
		gsc_label_index("AI Fsamp Range", index);

	ret		= aiss16ao2_ioctl(fd, AISS16AO2_IOCTL_AI_FSAMP_RANGE, &set);
	errs	= ret ? 1 : 0;

	switch (set)
	{
		default:

			errs++;
			sprintf(buf, "Unrecognized option: 0x%lX", (long) set);
			break;

		case AISS16AO2_FSAMP_RANGE_LOW:

			ptr	= "Low (0Hz-800KHz)";
			break;

		case AISS16AO2_FSAMP_RANGE_HIGH:

			ptr	= "High (1KHz-1MHz)";
			break;
	}

	if (verbose)
		printf("%s  (%s)\n", errs ? "FAIL <---" : "PASS", ptr);

	if (get)
		get[0]	= set;

	return(errs);
}


