// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AISS16AO2/utils/util_ai_range_a.c $
// $Rev: 45208 $
// $Date: 2019-07-03 17:20:23 -0500 (Wed, 03 Jul 2019) $

// 16AISS16AO2: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	aiss16ao2_ai_range_a
*
*	Purpose:
*
*		Provide a visual wrapper for the AISS16AO2_IOCTL_AI_RANGE_B service.
*
*	Arguments:
*
*		fd		The handle to use to access the driver.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		set		This is the value to apply.
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
******************************************************************************/

int	aiss16ao2_ai_range_a(int fd, int index, int verbose, s32 set, s32* get)
{
	char		buf[128];
	int			errs;
	const char*	ptr		= buf;
	int			ret;

	if (verbose)
		gsc_label_index("AI Range A", index);

	ret		= aiss16ao2_ioctl(fd, AISS16AO2_IOCTL_AI_RANGE_A, &set);
	errs	= ret ? 1 : 0;

	switch (set)
	{
		default:

			errs++;
			sprintf(buf, "Unrecognized option: 0x%lX", (long) set);
			break;

		case AISS16AO2_RANGE_2_5V:

			ptr	= "+- 2.5 V";
			break;

		case AISS16AO2_RANGE_5V:

			ptr	= "+- 5 V";
			break;

		case AISS16AO2_RANGE_10V:

			ptr	= "+- 10 V";
			break;
	}

	if (verbose)
		printf("%s  (%s)\n", errs ? "FAIL <---" : "PASS", ptr);

	if (get)
		get[0]	= set;

	return(errs);
}


