// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AISS16AO2/utils/util_config_ao.c $
// $Rev: 52934 $
// $Date: 2023-04-20 15:50:26 -0500 (Thu, 20 Apr 2023) $

// 16AISS16AO2: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	aiss16ao2_config_ao
*
*	Purpose:
*
*		Configure the given device's analog output options.
*
*	Arguments:
*
*		fd		The handle to use to access the driver.
*
*		index	The index of the device to access. Ignore if < 0.
*
*		verbose	Work verbosely?
*
*		fsamp	This is the desired Fsamp rate, of -1 to use the default.
*
*		master	The master clock frequency, or -1.
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
******************************************************************************/

int aiss16ao2_config_ao(int fd, int index, int verbose, s32 fsamp, s32 master_clock)
{
	int	errs	= 0;
	s32	ndiv;

	if (verbose)
	{
		gsc_label_index("Output Configuration", index);
		printf("\n");
		gsc_label_level_inc();
	}

	errs	+= aiss16ao2_initialize			(fd, index, verbose);

	// Settings must be applied after the initialization call!

	errs	+= aiss16ao2_tx_io_mode			(fd, index, verbose, GSC_IO_MODE_PIO,					NULL);
	errs	+= aiss16ao2_tx_io_overflow		(fd, index, verbose, AISS16AO2_IO_OVERFLOW_CHECK,		NULL);
	errs	+= aiss16ao2_tx_io_timeout		(fd, index, verbose, 30,								NULL);

	errs	+= aiss16ao2_master_clock		(fd, index, verbose, master_clock,						NULL);
	errs	+= aiss16ao2_data_format		(fd, index, verbose, AISS16AO2_DATA_FORMAT_OFF_BIN,		NULL);

	errs	+= aiss16ao2_ao_clock_mode		(fd, index, verbose, AISS16AO2_CLOCK_MODE_OUT,			NULL);
	errs	+= aiss16ao2_ao_mode			(fd, index, verbose, AISS16AO2_AO_MODE_FIFO,			NULL);
	errs	+= aiss16ao2_ao_buf_mode		(fd, index, verbose, AISS16AO2_BUF_MODE_OPEN,			NULL);
	errs	+= aiss16ao2_ao_output_mode		(fd, index, verbose, AISS16AO2_AO_OUTPUT_MODE_SIMUL,	NULL);
	errs	+= aiss16ao2_ao_thresh_lvl		(fd, index, verbose, 200L * 1024,						NULL);
	errs	+= aiss16ao2_ao_burst_enable	(fd, index, verbose, AISS16AO2_BURST_ENABLE_NO,			NULL);
	errs	+= aiss16ao2_ao_chan_sel		(fd, index, verbose, 0x3,								NULL);
	errs	+= aiss16ao2_ao_range			(fd, index, verbose, AISS16AO2_RANGE_10V,				NULL);

	errs	+= aiss16ao2_trigger_mode		(fd, index, verbose, AISS16AO2_TRIGGER_MODE_OUT,		NULL);
	errs	+= aiss16ao2_fsamp_ao_compute	(fd, index, verbose, fsamp, &ndiv,						NULL);
	errs	+= aiss16ao2_gen_c_ndiv			(fd, index, verbose, ndiv,								NULL);
	errs	+= aiss16ao2_gen_c_enable		(fd, index, verbose, AISS16AO2_GEN_ENABLE_YES,			NULL);
	errs	+= aiss16ao2_fsamp_ao_report_all(fd, index, verbose,									NULL);

	// Settings affecting the sample rate must be applied before autocalibration.
	// Refer to the board user manual for clarification.

//	errs	+= aiss16ao2_auto_calibrate		(fd, index, verbose);	// cable/equipment may need to be disconnected

	// Interrupt selection should be applied after autocalibration.
	// If detecting Autocalibration Done, then apply the setting beforehand.
	errs	+= aiss16ao2_irq_enable			(fd, index, verbose, 0,									NULL);

	errs	+= aiss16ao2_ao_enable			(fd, index, verbose, AISS16AO2_AIO_ENABLE_YES,			NULL);
	errs	+= aiss16ao2_ao_buf_clear		(fd, index, verbose);
	errs	+= aiss16ao2_ao_buf_over_data	(fd, index, verbose, AISS16AO2_BUF_OVERFLOW_CLEAR,		NULL);
	errs	+= aiss16ao2_ao_buf_over_fram	(fd, index, verbose, AISS16AO2_BUF_OVERFLOW_CLEAR,		NULL);
	errs	+= aiss16ao2_ao_buf_level		(fd, index, verbose,									NULL);

	if (verbose)
		gsc_label_level_dec();

	return(errs);
}


