// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AISS16AO2/utils/util_tx_io_timeout.c $
// $Rev: 52934 $
// $Date: 2023-04-20 15:50:26 -0500 (Thu, 20 Apr 2023) $

// 16AISS16AO2: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	aiss16ao2_tx_io_timeout
*
*	Purpose:
*
*		Provide a visual wrapper for the AISS16AO2_IOCTL_TX_IO_TIMEOUT service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		set		This is the value to apply.
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int	aiss16ao2_tx_io_timeout(int fd, int index, int verbose, s32 set, s32* get)
{
	char		buf[128];
	int			errs;
	const char*	ptr;
	int			ret;

	if (verbose)
		gsc_label_index("Tx I/O Timeout", index);

	ret		= aiss16ao2_ioctl(fd, AISS16AO2_IOCTL_TX_IO_TIMEOUT, &set);
	errs	= ret ? 1 : 0;

	if (verbose)
	{
		if (set == -1)
		{
			ptr	= "This service is not supported by this device.";
		}
		else if (set == AISS16AO2_IO_TIMEOUT_INFINITE)
		{
			ptr	= "infinite/never timeout";
		}
		else
		{
			sprintf(buf, "%ld second%s", (long) set, (set == 1) ? "" : "s");
			ptr	= buf;
		}

		printf("%s  (%s)\n", errs ? "FAIL <---" : "PASS", ptr);
	}

	if (get)
		get[0]	= set;

	return(errs);
}


